package org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.BetweennessCentralityCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.CentralityAnalysis;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.CentralityResults;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.ClosenessCentralityCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.DegreeCentralityCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.centrality.PageRankCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.stability.StabilityAnalysis;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.stability.StabilityCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.stability.StabilityResults;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.algorithms.transitiveclosure.TransitiveClosureDFS;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.entity.Edge;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.entity.SimpleEdge;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.entity.Vertex;

import edu.uci.ics.jung.graph.DirectedGraph;

public class GraphAnalyzer<V,E> {
	
	private CentralityAnalysis centralityAnalysis;
	private StabilityAnalysis stabilityAnalysis;
	
	private DirectedGraph<Vertex, Edge> graph;
	
	public GraphAnalyzer() {
	}

	public GraphAnalyzer(DirectedGraph<Vertex, Edge> graph) {
		this.graph = graph;
	}

	public DirectedGraph<Vertex, Edge> getGraph() {
		return graph;
	}

	public void setGraph(DirectedGraph<Vertex, Edge> graph) {
		this.graph = graph;
		this.centralityAnalysis = null;
		this.stabilityAnalysis = null;
	}

	
	public StabilityAnalysis calculateStabilityAnalysis(){
		
		if (this.stabilityAnalysis == null) {
			StabilityCalculator calculator = new StabilityCalculator();
			double overallStability = calculator.calculateOverallStability(this.graph);
			this.stabilityAnalysis = new StabilityResults(overallStability);
		}
		return this.stabilityAnalysis;
	}
	
	public CentralityAnalysis calculateCentralityAnalysis(){
		
		if (this.graph == null)
			throw new IllegalStateException("The graph should not be null");		
		
		if (this.centralityAnalysis != null)
			return this.centralityAnalysis;
			
		CentralityResults centralityResults = new CentralityResults(graph);
		
		DegreeCentralityCalculator degreeCalc = new DegreeCentralityCalculator();
		centralityResults.setVerticesDegreeCentrality(degreeCalc.calculateVerticesDegreeCentrality(this.graph));
		centralityResults.setGraphDegreeCentrality(degreeCalc.calculateGraphDegreeCentrality(this.graph));
		
		ClosenessCentralityCalculator closenessCalc = new ClosenessCentralityCalculator();		
		centralityResults.setVerticesClosenessCentrality(closenessCalc.calculateVerticesClosenessCentrality(this.graph));
		
		BetweennessCentralityCalculator betweenessCalc = new BetweennessCentralityCalculator();				
		centralityResults.setVerticesBetweenessCentrality(betweenessCalc.calculateVerticesBetweennessCentrality(this.graph));
		
		PageRankCalculator pageRankCalc = new PageRankCalculator();
		centralityResults.setVerticesPageRank(pageRankCalc.calculatePageRank(graph));
		
		this.centralityAnalysis = centralityResults;
		return centralityResults;
	}
	
	public DirectedGraph<V, SimpleEdge> getTransitiveClosureFrom(
			DirectedGraph<V, E> graph) {
		
		TransitiveClosureDFS<V, E> calculator = 
				new TransitiveClosureDFS<V, E>();
		
		return calculator.getTransitiveClosureFrom(graph);
	}
	
}