package org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.traversal;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;

/**
 * Non-recursive Breadth-First Search (BFS)
 * @author goliva@ime.usp.br
 *
 * @param <V> Vertex type
 * @param <E> Edge type
 */
public class NonRecursiveBFS<V,E> {

	private List<V> visited = 
			new LinkedList<V>();
	
	protected void traverse(Graph<V,E> graph){
		
		for(V v : graph.getVertices()){
			if (!visited.contains(v)){
				traverse(graph, v);
			}
		}
	}
	
	protected void traverse(Graph<V,E> graph, V root){
		
		Queue<V> deque = new LinkedList<V>();
		deque.add(root);
		
		while(!deque.isEmpty()){
			V v = deque.poll();
			if (!visited.contains(v)){
				
				//Template method
				visit(root, v);
				
				//Mark as visited
				visited.add(v);
				
				for (V w : graph.getSuccessors(v)){
					if (!visited.contains(w)){
						deque.add(w);
					}
				}
			}
		}
	}
	
	public void reset(){
		visited.clear();
	}
	
	protected void visit(V root, V v){
		System.out.println("Visited: " + v);
	}
	
	public static void main(String[] args) {
		DirectedGraph<String, Integer> graph = 
				new DirectedSparseGraph<String, Integer>();
		
		String s = new String("s");
		String a = new String("a");
		String b = new String("b");
		String c = new String("c");
		String e = new String("e");
		String f = new String("f");
		String g = new String("g");
		String h = new String("h");
		String i = new String("i");
		
		graph.addEdge(1, s, a);
		graph.addEdge(2, s, c);
		graph.addEdge(3, a, e);
		graph.addEdge(4, b, a);
		graph.addEdge(5, c, g);
		graph.addEdge(6, c, e);
		graph.addEdge(7, e, b);
		graph.addEdge(8, e, h);
		graph.addEdge(9, f, c);
		graph.addEdge(10, g, f);
		graph.addEdge(11, g, i);
		graph.addEdge(12, g, h);
		graph.addEdge(13, h, g);
		graph.addEdge(14, i, h);
		
		NonRecursiveBFS<String, Integer> bfs = 
				new NonRecursiveBFS<String, Integer>();
		
		bfs.traverse(graph,s);
	}
	
}
