package org.ow2.choreos.chordevelopment.analysis.depanalyzer.graph.traversal;

import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;

/**
 * Non-recursive Depth-First Search (DFS)
 * @author goliva@ime.usp.br
 *
 * @param <V> Vertex type
 * @param <E> Edge type
 */
public class NonRecursiveDFS<V,E> {

	private List<V> visited = 
			new LinkedList<V>();
	
	protected void traverse(Graph<V,E> graph){
		
		for(V v : graph.getVertices()){
			if (!visited.contains(v)){
				traverse(graph, v);
			}
		}
	}
	
	protected void traverse(Graph<V,E> graph, V root){
		
		Stack<V> stack = new Stack<V>();
		stack.push(root);
		
		while(!stack.isEmpty()){
			V v = stack.pop();
			if (!visited.contains(v)){
				
				//Template method
				visit(root, v);
				
				//Mark as visited
				visited.add(v);
				
				for (V w : graph.getSuccessors(v)){
					if (!visited.contains(w)){
						stack.push(w);
					}
				}
			}
		}
	}
	
	public void reset(){
		visited.clear();
	}
	
	protected void visit(V root, V v){
		System.out.println("Visited: " + v);
	}
	
	public static void main(String[] args) {
		DirectedGraph<String, Integer> graph = 
				new DirectedSparseGraph<String, Integer>();
		
		String u = new String("u");
		String v = new String("v");
		String w = new String("w");
		String x = new String("x");
		String y = new String("y");
		String z = new String("z");
		
		graph.addEdge(1, u, v);
		graph.addEdge(2, u, x);
		graph.addEdge(3, v, y);
		graph.addEdge(4, w, y);
		graph.addEdge(5, w, z);
		graph.addEdge(6, x, v);
		graph.addEdge(7, y, x);
		graph.addEdge(8, z, z);
		
		NonRecursiveDFS<String, Integer> dfs = 
				new NonRecursiveDFS<String, Integer>();
		
		dfs.traverse(graph);
	}
}