package org.ow2.choreos.chordevelopment.analysis.depanalyzer.parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Choreography;

import com.ebmwebsourcing.easybox.api.XmlContext;
import com.ebmwebsourcing.easybox.api.XmlContextFactory;
import com.ebmwebsourcing.easybox.api.XmlObjectReadException;
import com.ebmwebsourcing.easybox.api.XmlObjectReader;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.Definitions;

public class LinagoraChoreographyParser {

	public Set<Choreography> parseChoreographies(List<String> xmiFiles){
		HashSet<Choreography> choreographies = new HashSet<Choreography>();
		for (String xmiFile : xmiFiles){
			Choreography choreography = parseChoreography(xmiFile);
			choreographies.add(choreography);
		}
		return choreographies;
	}
	
	public Choreography parseChoreography(String xmiFile){
		
		try {
			Definitions chorDefinitions = readDefinitions(xmiFile);
			
			LinagoraParser2DomainModelAdaptor adaptor = 
					new LinagoraParser2DomainModelAdaptor();
			
			Choreography choreography = adaptor.adapt(chorDefinitions);
			
			return choreography;
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (XmlObjectReadException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public Definitions readDefinitions(String xmiFile) throws FileNotFoundException, 
		XmlObjectReadException{
		
		XmlContextFactory xmlContextFactory = new XmlContextFactory();
		XmlContext xmlContext = xmlContextFactory.newContext();
		XmlObjectReader reader = xmlContext.createReader();
		
		Definitions definitions = reader.readDocument(
				new FileInputStream(new File(xmiFile)),	Definitions.class);
		
		return definitions;
	
	}	
	
	public static void main(String[] args) {
		LinagoraChoreographyParser linagoraParser = 
				new LinagoraChoreographyParser();
		
		try{
			Choreography definitions = 
					linagoraParser.parseChoreography("Data.bpmn2");
			
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
}