package org.ow2.choreos.chordevelopment.analysis.depanalyzer.parser;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Choreography;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.FinalStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Participant;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.StartStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Step;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.TaskStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Transition;

import com.ebmwebsourcing.easybpmn.bpmn20.api.element.ChoreographyTask;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.Collaboration;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.Definitions;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.FlowNode;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.Message;
import com.ebmwebsourcing.easybpmn.bpmn20.api.element.MessageFlow;

public class LinagoraParser2DomainModelAdaptor {

	public LinagoraParser2DomainModelAdaptor(){
		
	}
	
	public Choreography adapt(Definitions definitions){
		
		
		Choreography choreography = parseChoreography(definitions);
		
		//Participants
		Set<Participant> participants = parseParticipants(definitions);		
		choreography.setParticipants(participants);
		
		//Steps
		Set<Step> steps = parseSteps(choreography, definitions);
		choreography.setSteps(steps);
		
		//Transitions
		Set<Transition> transitions = parseTransitions(
				choreography, definitions);
		
		return choreography;
	}

	private Choreography parseChoreography(Definitions definitions) {
		for(com.ebmwebsourcing.easybpmn.bpmn20.api.element.Choreography
				rawChoreography : definitions.getChoreographies()){
			
			Choreography choreography = new Choreography(
					rawChoreography.getId(), rawChoreography.getName());
			
			return choreography;
		}
		
		return null;		
	}

	private Set<Participant> parseParticipants(Definitions chorDefinitions) {
		Set<Participant> participants = new HashSet<Participant>();
		for(Collaboration collab : chorDefinitions.getCollaborations()){
			for(com.ebmwebsourcing.easybpmn.bpmn20.api.element.Participant 
					rawParticipant : collab.getParticipant()){
				
				Participant participant = new Participant(
						rawParticipant.getId(), rawParticipant.getName());
				
				participants.add(participant);		
			}
		}
		return participants;
	}
	
	private Set<Step> parseSteps(Choreography choreography, 
			Definitions definitions) {
		
		Set<Step> steps = new HashSet<Step>();
		
		for(com.ebmwebsourcing.easybpmn.bpmn20.api.element.Choreography
				rawChoreography : definitions.getChoreographies()){
			
			for(FlowNode flowNode : rawChoreography.getFlowNode()){
				String stepType = flowNode.getXmlObjectQName().getLocalPart();
				if ("startEvent".equals(stepType)){ 
						StartStep startStep = parseStartStep(flowNode);
						steps.add(startStep);
				} else if ("endEvent".equals(stepType)){
						FinalStep finalStep = parseFinalStep(flowNode);
						steps.add(finalStep);
				} else if ("choreographyTask".equals(stepType)){
						ChoreographyTask choreographyTask = 
							(ChoreographyTask)flowNode;
						
						TaskStep taskStep = parseTaskStep(choreography, 
								definitions,choreographyTask);
						steps.add(taskStep);
				}
			}
		}
		return steps;
	}

	private TaskStep parseTaskStep(Choreography choreography, 
			Definitions definitions, ChoreographyTask choreographyTask) {
		
		String ID = choreographyTask.getId();
		String name = choreographyTask.getName();
		
		String initiatingParticipantID = 
				choreographyTask.getInitiatingParticipantRef().getLocalPart();
						
		String targetParticipantID = null;
		for(QName qName : choreographyTask.getParticipantRef()){
			if(!qName.getLocalPart().equals(initiatingParticipantID)){
				targetParticipantID = qName.getLocalPart();
			}
		}
			
		//This is weird!
		System.out.println("Number of message flows in choreography: " + definitions.getChoreographies()[0].getMessageFlow().length);
		
		//Retrieving messages
		Map<String, Message> messageMap = new HashMap<String, Message>();
		for(Message m : definitions.getMessages()){
			messageMap.put(m.getId(), m);
		}
		
		String initiatingMessage = null;
		String responseMessage = null;
		
		for(Collaboration collaboration : definitions.getCollaborations()){
			
			for(MessageFlow messageFlow : collaboration.getMessageFlow()){
				
				//Initiating Message
				if(messageFlow.getSourceRef().equals(initiatingParticipantID)){
					initiatingMessage = messageMap.get(
							messageFlow.getMessageRef()).getName();
				}
				//Response Message
				else{
					responseMessage = messageMap.get(
							messageFlow.getMessageRef()).getName();
				}
			}
		}
		
		TaskStep taskStep = new TaskStep(ID, 
				choreography.getParticipant(initiatingParticipantID), 
				initiatingMessage, name, 
				choreography.getParticipant(targetParticipantID),
				responseMessage);
		
		return taskStep;
	}

	private FinalStep parseFinalStep(FlowNode flowNode) {
		FinalStep finalStep = new FinalStep(flowNode.getId());
		return finalStep;
	}

	private StartStep parseStartStep(FlowNode flowNode) {
		StartStep startStep = new StartStep(flowNode.getId());
		return startStep;
	}
	
	private Set<Transition> parseTransitions(Choreography choreography,
			Definitions definitions) {
		// TODO Auto-generated method stub
		return null;
	}
}
