package org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Choreography;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.FinalStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.GatewayStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.GatewayType;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Participant;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.StartStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Step;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.SubChoreography;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.SubchoreographyStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.TaskStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Transition;

public class AuctionHouseChoreography extends Choreography{

	public AuctionHouseChoreography(){
		super("#AuctionHouseChoreography#", "AuctionHouse Choreography");
		
		Set<Participant> participants = buildParticipants();
		this.setParticipants(participants);
		
		Set<Step> steps = buildSteps(this);
		this.setSteps(steps);
		
		Set<Transition> transitions = buildTransitions(this);
		this.setTransitions(transitions);
	}	

	public Set<Participant> buildParticipants(){
		Participant seller = new Participant("#Seller#", "Seller");
		
		Participant auctionHouse = new Participant("#AuctionHouse#", 
				"Auction House");
		
		Participant potentialBuyer = new Participant("#PotentialBuyer#", 
				"Potential Buyer");
		
		Participant buyer = new Participant("#Buyer#", "Buyer");
		
		HashSet<Participant> participants = new HashSet<Participant>();
		participants.add(seller);
		participants.add(auctionHouse);
		participants.add(potentialBuyer);
		participants.add(buyer);
		
		return participants;
	}
	
	private Set<Step> buildSteps(Choreography choreography) {
		Set<Step> steps = new HashSet<Step>();
		
		steps.add(buildStartStep(choreography));
		steps.addAll(buildFinalSteps(choreography));
		steps.add(buildGatewaySteps(choreography));
		
		steps.addAll(buildTaskSteps(choreography));
		steps.addAll(buildSubChorSteps(choreography));
		
		return steps;
	}

	private Step buildStartStep(Choreography choreography) {
		StartStep startStep = new StartStep("#StartStep#");
		return startStep;
	}
	
	private Set<Step> buildFinalSteps(Choreography choreography) {
		FinalStep happyFinalStep = new FinalStep("#HappyFinalStep#");
		FinalStep badFinalStep = new FinalStep("#BadFinalStep#");
		Set<Step> finalSteps = new HashSet<Step>();
		finalSteps.add(happyFinalStep);
		finalSteps.add(badFinalStep);
		return finalSteps;
	}
	
	private Step buildGatewaySteps(Choreography choreography) {
		GatewayStep exclusiveGateway = new GatewayStep(
				"#Gateway#",GatewayType.Exclusive); 
				
		return exclusiveGateway;
	}

	private Set<Step> buildTaskSteps(Choreography choreography) {
		Set<Step> taskSteps = new HashSet<Step>();
		
		TaskStep publishAuction = buildPublishAuctionTask(choreography);
		
		TaskStep informAboutProductBeingNotSold = 
				buildInformAboutProductBeingNotSoldTask(choreography);
		
		taskSteps.add(publishAuction);
		taskSteps.add(informAboutProductBeingNotSold);
		
		return taskSteps;
	}

	private TaskStep buildPublishAuctionTask(Choreography choreography) {
		
		//Publish auction
		String ID = "#PublishAuction#";
		Participant initiatingParticipant = 
				choreography.getParticipantByName("Seller");
		String initiatingMessage = "product information";
		String name = "publish auction";
		Participant targetParticipant = 
				choreography.getParticipantByName("Auction House");
		String responseMessage = "auction no.";
		
		TaskStep publishAuction  = new TaskStep(ID, initiatingParticipant,
				initiatingMessage, name, targetParticipant, responseMessage);
		
		return publishAuction;
	}
	
	private TaskStep buildInformAboutProductBeingNotSoldTask(
			Choreography choreography) {
		
		//Inform about product being not sold
		String ID = "#InformAboutProductBeingNotSold#";
		Participant initiatingParticipant = 
				choreography.getParticipantByName("Auction House");
		String initiatingMessage = new String();
		String name = "inform about product being not sold";
		Participant targetParticipant = 
				choreography.getParticipantByName("Seller");
		String responseMessage = new String();
		
		TaskStep informAboutProductBeingNotSold  = new TaskStep(ID, 
				initiatingParticipant, initiatingMessage, name, 
				targetParticipant, responseMessage);
		
		return informAboutProductBeingNotSold;
	}
	
	private Collection<Step> buildSubChorSteps(Choreography choreography) {
		
		//Place offer
		SubchoreographyStep placeOfferSubChor = new SubchoreographyStep(
				"#PlaceOffer#",	new SubChoreography("#PlaceOfferSubchor#", 
				"place offer", choreography), false);
		
		//Purchase Process
		SubchoreographyStep purchaseProcessSubChor = new SubchoreographyStep(
				"#PurchaseProcess#", new SubChoreography(
				"#PurchaseProcessSubchor#", "purchase process", choreography), 
				false);
		
		Set<Step> subChorSteps = new HashSet<Step>();
				
		subChorSteps.add(placeOfferSubChor);
		subChorSteps.add(purchaseProcessSubChor);
		
		return subChorSteps;
	}

	private Set<Transition> buildTransitions(Choreography choreography) {
		
		Step startStep = choreography.getStep("#StartStep#");
		Step publishAuction = choreography.getStep("#PublishAuction#");
		Step placeOffer = choreography.getStep("#PlaceOffer#");
		Step gateway = choreography.getStep("#Gateway#");
		Step purchaseProcess = choreography.getStep("#PurchaseProcess#");
		Step happyFinal = choreography.getStep("#HappyFinalStep#");
		
		Step productNotSold = choreography.getStep(
				"#InformAboutProductBeingNotSold#");
		
		Step badFinal = choreography.getStep("#BadFinalStep#");
		
		Transition t1 = new Transition(
				"t1", new String(), startStep, publishAuction);
		Transition t2 = new Transition(
				"t2", new String(), publishAuction, placeOffer);
		Transition t3 = new Transition(
				"t3", new String(), placeOffer, gateway);
		Transition t4 = new Transition(
				"t4", "product sold", gateway, purchaseProcess);
		Transition t5 = new Transition(
				"t5", new String(), purchaseProcess, happyFinal);
		Transition t6 = new Transition(
				"t6", "product not sold", gateway, productNotSold);
		Transition t7 = new Transition(
				"t7", new String(), productNotSold, badFinal);
		
		Set<Transition> transitions = new HashSet<Transition>();
		transitions.add(t1);
		transitions.add(t2);
		transitions.add(t3);
		transitions.add(t4);
		transitions.add(t5);
		transitions.add(t6);
		transitions.add(t7);
		
		return transitions;
	}
	
}