#include<iostream>
#include "OperationGraph.h"
using namespace std;

OperationGraph :: OperationGraph(){
	size = 0;
	Order = NULL;
	ordersize = 0;
}


OperationGraph :: ~OperationGraph(){
	if (Order != NULL) {
		delete[] Order;
	}

}

void OperationGraph :: loadGraph(char* modelfile, char *qosfile, char *probfile){

	int n, nc;
	int id, i, j, k, l;
	string strType;
	int type;
	float srt, ec, pj, ava, rel;
	
	BPMNUnitTask* task;
	

	ifstream modfile (modelfile, ifstream::in);
	ifstream datfile(qosfile, ifstream::in);
	ifstream pfile(probfile, ifstream::in);
	
	modfile >> n; // n is the number of nodes in the model.
	if(n > 0){
		size = n;
		Nodes = new BPMNUnitTask*[size];
	}
	// Read nodes, SRT and EC
	for(i = 0; i < n; i++){
		modfile >> id >> strType;
		task = new BPMNUnitTask();
		task->setId(id);
		Nodes[i] = task;
		task = NULL;
		
		if(strType.compare("OPERATION") == 0){
			datfile >> srt >> ec >> ava >> rel;
			Nodes[i]->setSrt(srt);
			Nodes[i]->setEc(ec);
			Nodes[i]->setAva(ava);
			Nodes[i]->setRel(rel);
			
			Nodes[i]->setType(OPERATION); 
		}
		else{ 
			if(strType.compare("XOR_SPLIT") == 0)
				Nodes[i]->setType(XOR_SPLIT); 
			else
			if(strType.compare("OR_SPLIT") == 0)
				Nodes[i]->setType(OR_SPLIT); 
			else
			if(strType.compare("AND_SPLIT") == 0)
				Nodes[i]->setType(AND_SPLIT); 
			else
			if(strType.compare("XOR_JOIN") == 0)
				Nodes[i]->setType(XOR_JOIN); 
			else
			if(strType.compare("OR_JOIN") == 0)
				Nodes[i]->setType(OR_JOIN); 	
			else
			if(strType.compare("AND_JOIN") == 0)
				Nodes[i]->setType(AND_JOIN); 			
	
			Nodes[i]->setSrt(0); // The srt for connector is NULL;
			Nodes[i]->setEc(0);
		}	
	}
	// Read the successors ( we know that there is at least one task without successor
	modfile >> n; // Read the number of tasks for which a sucessor will be defined.

	for(i = 0; i < n; i++){
		modfile >> nc;
		modfile >> id;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;

		Nodes[k]->succNumber = nc;
		Nodes[k]->succs = new BPMNUnitTask*[nc];
		for(j = 0; j < nc; j++){
			modfile >> id;
			l = 0;
			while(Nodes[l]->getId() != id)
				l++;
			Nodes[k]->succs[j] = Nodes[l];
			if (isOpen(Nodes[k]) && (Nodes[k]->getType() != AND_SPLIT)) {
				pfile >> pj;
				Nodes[k]->setProbability(j,pj);
			}
			
			Nodes[l]->setPred(Nodes[k]);

		}
		if (isOpen(Nodes[k]) && (Nodes[k]->getType() == OR_SPLIT)){
			pfile >> pj;
			Nodes[k]->setProbability(nc,pj);
		}
	}

	modfile >> n; // read the number of loop connectors
	int nl;
	for(i = 0; i < n; i++){
		modfile >> id >> nc;
		datfile >> nl;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		l = 0;
		while(Nodes[l]->getId() != nc)
			l++;
		Nodes[k]->predLoop = Nodes[l];
		Nodes[k]->nloop = nl;
		Nodes[l]->loop_starter = TRUE;
	}
	modfile.close();
	datfile.close();
	pfile.close();

}



void OperationGraph :: reloadGraph(char* modelfile, char *qosfile, char *probfile){
	
	int n, nc;
	int id, i, j, k, l;
	string strType;
	int type;
	float srt, ec, pj, ava, rel;
		

	ifstream modfile (modelfile, ifstream::in);
	ifstream datfile(qosfile, ifstream::in);
	ifstream pfile(probfile, ifstream::in);
	
	modfile >> n; // n is the number of nodes in the model.
	if(n > 0){
		size = n;
	}
	// Read nodes, SRT and EC
	
	for(i = 0; i < n; i++){
		modfile >> id >> strType;
		Nodes[i]->init();
		Nodes[i]->setId(id);
		
		if(strType.compare("OPERATION") == 0){
			datfile >> srt >> ec >> ava >> rel;
			Nodes[i]->setSrt(srt);
			Nodes[i]->setEc(ec);
			Nodes[i]->setAva(ava);
			Nodes[i]->setRel(rel);
			
			Nodes[i]->setType(OPERATION); 
		}
		else{ 
			if(strType.compare("XOR_SPLIT") == 0)
				Nodes[i]->setType(XOR_SPLIT); 
			else
				if(strType.compare("OR_SPLIT") == 0)
					Nodes[i]->setType(OR_SPLIT); 
				else
					if(strType.compare("AND_SPLIT") == 0)
						Nodes[i]->setType(AND_SPLIT); 
					else
						if(strType.compare("XOR_JOIN") == 0)
							Nodes[i]->setType(XOR_JOIN); 
						else
							if(strType.compare("OR_JOIN") == 0)
								Nodes[i]->setType(OR_JOIN); 	
							else
								if(strType.compare("AND_JOIN") == 0)
									Nodes[i]->setType(AND_JOIN); 			
			
			Nodes[i]->setSrt(0); // The srt for connector is NULL;
			Nodes[i]->setEc(0);
		}	
	}
	
	// Read the successors ( we know that there is at least one task without successor
	modfile >> n; // Read the number of tasks for which a sucessor will be defined.
	
	for(i = 0; i < n; i++){
		modfile >> nc >> id;

		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		
		Nodes[k]->succNumber = nc;
//		Nodes[k]->succs = new BPMNUnitTask*[nc];
		for(j = 0; j < nc; j++){
			modfile >> id;
			l = 0;
			while(Nodes[l]->getId() != id)
				l++;
			Nodes[k]->succs[j] = Nodes[l];
			if (isOpen(Nodes[k]) && (Nodes[k]->getType() != AND_SPLIT) ) {
				pfile >> pj;
				Nodes[k]->setProbability(j,pj);
			}
			Nodes[l]->setPred(Nodes[k]);
			
		}
		if (isOpen(Nodes[k]) && (Nodes[k]->getType() == OR_SPLIT)){
			pfile >> pj;
			Nodes[k]->setProbability(nc,pj);
		}
	}

	
	modfile >> n; // read the number of loop connectors
	int nl;
	for(i = 0; i < n; i++){
		modfile >> id >> nc;
		datfile >> nl;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		l = 0;
		while(Nodes[l]->getId() != nc)
			l++;
		Nodes[k]->predLoop = Nodes[l];
		Nodes[k]->nloop = nl;
		Nodes[l]->loop_starter = TRUE;
	}
	modfile.close();
	datfile.close();
	pfile.close();

	
}



void OperationGraph :: loadGraphSrt(char* modelfile, char *qosfile, char *probfile, float powerMin, float powerMax){

	int n, nc;
	int id, i, j, k, l;
	string strType;
	int type;
	float srt, ec, pj;
	
	BPMNUnitTask* task;
	

	ifstream modfile (modelfile, ifstream::in);
	ifstream datfile(qosfile, ifstream::in);
	ifstream pfile(probfile, ifstream::in);
	
	modfile >> n; // n is the number of nodes in the model.
	if(n > 0){
		size = n;
		Nodes = new BPMNUnitTask*[size];
	}
	// Read nodes, SRT and EC
	for(i = 0; i < n; i++){
		modfile >> id >> strType;
		task = new BPMNUnitTask();
		task->setId(id);
		Nodes[i] = task;
		task = NULL;
		
		if(strType.compare("OPERATION") == 0){
			datfile >> srt;
			Nodes[i]->setSrt(srt);
			ec = (powerMin + rand()%( (int)(powerMax-powerMin)) )*srt/100;
			Nodes[i]->setEc(ec);
			Nodes[i]->setType(OPERATION); 
		}
		else{ 
			if(strType.compare("XOR_SPLIT") == 0)
				Nodes[i]->setType(XOR_SPLIT); 
			else
			if(strType.compare("OR_SPLIT") == 0)
				Nodes[i]->setType(OR_SPLIT); 
			else
			if(strType.compare("AND_SPLIT") == 0)
				Nodes[i]->setType(AND_SPLIT); 
			else
			if(strType.compare("XOR_JOIN") == 0)
				Nodes[i]->setType(XOR_JOIN); 
			else
			if(strType.compare("OR_JOIN") == 0)
				Nodes[i]->setType(OR_JOIN); 	
			else
			if(strType.compare("AND_JOIN") == 0)
				Nodes[i]->setType(AND_JOIN); 			
	
			Nodes[i]->setSrt(0); // The srt for connector is NULL;
			Nodes[i]->setEc(0);
		}	
	}
	// Read the successors ( we know that there is at least one task without successor
	modfile >> n; // Read the number of tasks for which a sucessor will be defined.

	for(i = 0; i < n; i++){
		modfile >> nc;
		modfile >> id;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;

		Nodes[k]->succNumber = nc;
		Nodes[k]->succs = new BPMNUnitTask*[nc];
		for(j = 0; j < nc; j++){
			modfile >> id;
			l = 0;
			while(Nodes[l]->getId() != id)
				l++;
			Nodes[k]->succs[j] = Nodes[l];
			if (isOpen(Nodes[k]) && (Nodes[k]->getType() != AND_SPLIT)) {
				pfile >> pj;
				Nodes[k]->setProbability(j,pj);
			}
			
			Nodes[l]->setPred(Nodes[k]);

		}
		if (isOpen(Nodes[k]) && (Nodes[k]->getType() == OR_SPLIT)){
			pfile >> pj;
			Nodes[k]->setProbability(nc,pj);
		}
	}

	modfile >> n; // read the number of loop connectors
	int nl;
	for(i = 0; i < n; i++){
		modfile >> id >> nc;
		datfile >> nl;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		l = 0;
		while(Nodes[l]->getId() != nc)
			l++;
		Nodes[k]->predLoop = Nodes[l];
		Nodes[k]->nloop = nl;
		Nodes[l]->loop_starter = TRUE;

	}
	modfile.close();
	datfile.close();
	pfile.close();


}



void OperationGraph :: reloadGraphSrt(char* modelfile, char *qosfile, char *probfile, float powerMin, float powerMax){
	
	int n, nc;
	int id, i, j, k, l;
	string strType;
	int type;
	float srt, ec, pj;
		

	ifstream modfile (modelfile, ifstream::in);
	ifstream datfile(qosfile, ifstream::in);
	ifstream pfile(probfile, ifstream::in);
	
	modfile >> n; // n is the number of nodes in the model.
	if(n > 0){
		size = n;
	}
	// Read nodes, SRT and EC
	
	for(i = 0; i < n; i++){
		modfile >> id >> strType;
		Nodes[i]->init();
		Nodes[i]->setId(id);
		
		if(strType.compare("OPERATION") == 0){
			datfile >> srt;
			Nodes[i]->setSrt(srt);
			ec = (powerMin + rand()%( (int)(powerMax-powerMin)) )*srt/100;
			Nodes[i]->setEc(ec);
			Nodes[i]->setType(OPERATION); 
		}
		else{ 
			if(strType.compare("XOR_SPLIT") == 0)
				Nodes[i]->setType(XOR_SPLIT); 
			else
				if(strType.compare("OR_SPLIT") == 0)
					Nodes[i]->setType(OR_SPLIT); 
				else
					if(strType.compare("AND_SPLIT") == 0)
						Nodes[i]->setType(AND_SPLIT); 
					else
						if(strType.compare("XOR_JOIN") == 0)
							Nodes[i]->setType(XOR_JOIN); 
						else
							if(strType.compare("OR_JOIN") == 0)
								Nodes[i]->setType(OR_JOIN); 	
							else
								if(strType.compare("AND_JOIN") == 0)
									Nodes[i]->setType(AND_JOIN); 			
			
			Nodes[i]->setSrt(0); // The srt for connector is NULL;
			Nodes[i]->setEc(0);
		}	
	}
	
	// Read the successors ( we know that there is at least one task without successor
	modfile >> n; // Read the number of tasks for which a sucessor will be defined.
	
	for(i = 0; i < n; i++){
		modfile >> nc >> id;

		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		
		Nodes[k]->succNumber = nc;
//		Nodes[k]->succs = new BPMNUnitTask*[nc];
		for(j = 0; j < nc; j++){
			modfile >> id;
			l = 0;
			while(Nodes[l]->getId() != id)
				l++;
			Nodes[k]->succs[j] = Nodes[l];
			if (isOpen(Nodes[k]) && (Nodes[k]->getType() != AND_SPLIT) ) {
				pfile >> pj;
				Nodes[k]->setProbability(j,pj);
			}
			Nodes[l]->setPred(Nodes[k]);
			
		}
		if (isOpen(Nodes[k]) && (Nodes[k]->getType() == OR_SPLIT)){
			pfile >> pj;
			Nodes[k]->setProbability(nc,pj);
		}
	}

	
	modfile >> n; // read the number of loop connectors
	int nl;
	for(i = 0; i < n; i++){
		modfile >> id >> nc;
		datfile >> nl;
		k = 0;
		while(Nodes[k]->getId() != id)
			k++;
		l = 0;
		while(Nodes[l]->getId() != nc)
			l++;
		Nodes[k]->predLoop = Nodes[l];
		Nodes[k]->nloop = nl;
		Nodes[l]->loop_starter = TRUE;
	}
	modfile.close();
	datfile.close();
	pfile.close();

	
}

int OperationGraph :: getSize(){
	return size;
}


void OperationGraph :: printGraphId(){

	int i;
	cout << "PRINTING IDs" << endl;
	for(i = 0; i < size; i++)
		cout << "Node id: = " << Nodes[i]->getId() << endl;
}


void OperationGraph :: insert(nodeOrder v){
	
	int i, pos;
	i = 0; pos = ordersize;
	
	while (i < ordersize) {
		if(Order[i].leaf == v.root)
			pos = i;
/*		else{
			if(Order[i].root == v.leaf)
				pos = i+1;
		}	
*/
		i++;
	}

	if(ordersize > 0){
		for(i = ordersize; i >= pos; i--){
			Order[i+1].root = Order[i].root;
			Order[i+1].leaf = Order[i].leaf;
			Order[i+1].type_root = Order[i].type_root;
		}
	}

	Order[pos].root = v.root;
	Order[pos].leaf = v.leaf;
	Order[pos].type_root = v.type_root;

	ordersize = ordersize + 1;

}


float OperationGraph :: max(float a, float b){

	if(a < b)
		return b;
	else
		return a;
}



int OperationGraph ::isOpen(BPMNUnitTask *u){
	
	if((u->getType() == XOR_SPLIT) || (u->getType() == OR_SPLIT) || (u->getType() == AND_SPLIT))
		return TRUE;
	else
		return FALSE;
}

int OperationGraph ::isClosed(BPMNUnitTask *u){
	
	if((u->getType() == XOR_JOIN) || (u->getType() == OR_JOIN) || (u->getType() == AND_JOIN))
		return TRUE;
	else
		return FALSE;
}

int OperationGraph ::isOperation(BPMNUnitTask *u){

	if(u->getType() == OPERATION)
		return TRUE;
	else
		return FALSE;
}

void OperationGraph :: computeOrder(){
	
	BPMNStack *Sd, *St, *Se;
	BPMNUnitTask *u;

	int i;

	if(Order != NULL)
		delete[] Order;

	ordersize = 0;
	Order  = new nodeOrder[size];

	Sd = new BPMNStack;
	St = new BPMNStack;
	Se = new BPMNStack; 

	
	Nodes[0]->mark();
	Sd->push(Nodes[0]);

	for(i = 0; i < Nodes[0]->succNumber; i++)
		Sd->push(Nodes[0]->succs[i]);
	

	while(!Sd->isEmpty()){

		u = Sd->pop();
//		cout << "id = " << u->getId() << endl;

		if(isOperation(u) || u->isMarked2())
			orderOperation(u, Sd, St, Se);
		else{
			if( isOpen(u))
				orderOpen(u, Sd, St, Se);
			else{
				if(isClosed(u))
					orderClosed(u, Sd, St, Se);
			}
			
		}
		
	}

	delete Sd;
	delete St;
	delete Se;

}


void OperationGraph :: orderOperation(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se){

	nodeOrder v;
	BPMNUnitTask *next;

//	cout << "id = " << u->getId() << endl;
	if (u->isMarked2()) {
		v.root = u->getId();
		v.leaf = u->leaf->getId();	
		v.type_root = u->getType();
		insert(v);
	}else{
	if(u->isMarked()){
		if((u->succNumber > 0) && (!isClosed(u->succs[0])) ){
			v.root = u->getId();
			v.leaf = u->succs[0]->getId();	
			v.type_root = OPERATION;
			insert(v);
		}
	}
	else{
		if((u->succNumber > 0) && (isClosed(u->succs[0]))){
			Se->pushx(u->succs[0]);
			next = u->succs[0];
			if((next->succNumber > 0))
				St->pushx(next->succs[0]);
		}
		else{
			if(u->succNumber > 0){
				u->mark();
				Sd->push(u);
				Sd->push(u->succs[0]);
			}
		}
	}
	}
}


void OperationGraph :: orderOpen(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se){
	
	BPMNUnitTask *w, *v, *next; nodeOrder r;
	int i;


	if(u->isMarked()){
		w = Se->pop();

		r.root = u->getId();
		r.leaf = w->getId();	
		r.type_root = u->getType();
		insert(r);

		if((w->succNumber > 0) && (!isClosed(w->succs[0])) ){
			u->setleaf(w->succs[0]);
/*			r.root = u->getId();
			r.leaf = w->succs[0]->getId();
			r.type_root = u->getType();
			insert(r);
 */
			u->mark2();
			Sd->push(u);
		}
		
		if(!St->isEmpty()){
			v = St->pop();
			Sd->push(v);
		}
	
	}else{
		u->mark();
		Sd->push(u);
		for(i = 0; i < u->succNumber; i++)
			Sd->push(u->succs[i]);
	}

}

void OperationGraph :: orderClosed(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se){
	
	Se->pushx(u);
	if(u->succNumber > 0)
		St->pushx(u->succs[0]);
}


int OperationGraph :: getRootNode(int root){
	int i;
	i = 0;
	while( Nodes[i]->getId() != root)
		i++;
	return i;
}

void OperationGraph :: sequence_reduction(int root){
	
	BPMNUnitTask *next;	
	int i, j;

	// We get the position of the node to reduce
	i = getRootNode(root);
	// If there is not a successor, there is nothing to do
	if(Nodes[i]->succNumber > 0){

		// The next node will be deleted So we remove pointers towards it.
		next = Nodes[i]->succs[0];
		for(j = 0; j < next->predNumber; j++)
			next->preds[j]->removeSuccessor(next);

		// We update the QoS (sequence aggregation rules)
		Nodes[i]->setSrt(Nodes[i]->getSrt() + next->getSrt()); 
		Nodes[i]->setEc(Nodes[i]->getEc() + next->getEc());

		// We update the successors pointer and delete useless nodes			
		if(next->succNumber > 0){
			Nodes[i]->succNumber = 1;				
			Nodes[i]->succs[0] = next->succs[0];
			next->removeallSuccessors();
		}else{
			Nodes[i]->succNumber = 0;
			Nodes[i]->removeallSuccessors();
		}

		next->removeallPredecessors();
	}
	
}


void OperationGraph :: fork_reduction(int root, int leaf){

	BPMNUnitTask *next;	
	int i, pos;
	float srt, ec;	

	// We find the root pointer and initialize srt and ec	
	pos = getRootNode(root);
	srt = Nodes[pos]->getSrt();
	ec = Nodes[pos]->getEc();
	
	next = NULL;
	for(i = 0; i < Nodes[pos]->succNumber; i++){
		if((Nodes[pos]->succs[i]->succNumber < 0) || (Nodes[pos]->succs[i]->succs[0]->getId() != leaf)){
			cout << "Error in the specified leaf \n";
			return;
		}else{
		
			// QoS computation 		
			srt = max(srt,Nodes[pos]->succs[i]->getSrt());
			ec = ec + Nodes[pos]->succs[i]->getEc();

			if(next == NULL)
				next = Nodes[pos]->succs[0]->succs[0];
			// We prepare and delete the next node
			Nodes[pos]->succs[i]->removeallSuccessors();
			Nodes[pos]->succs[i]->removeallPredecessors();
			Nodes[pos]->succs[i] = NULL;
		}
	}

	// Link the And Split with the Join successor

	if(next->succNumber > 0){
		Nodes[pos]->succNumber = 1;
		Nodes[pos]->succs[0] = next->succs[0];
		next->succs[0]->removePredecessor(next);
		next->removeallSuccessors();
		next->removeallPredecessors();
	}else{
		Nodes[pos]->succNumber = 0;
		Nodes[pos]->succs[0] = NULL;
	}
	Nodes[pos]->setSrt(srt);
	Nodes[pos]->setEc(ec);
}



void OperationGraph :: xor_reduction(int root, int leaf){

	BPMNUnitTask *next;	
	int i, pos;
	float srt, ec;	

	// We find the root pointer and initialize srt and ec	


	pos = getRootNode(root);
	srt = Nodes[pos]->getSrt();
	ec = Nodes[pos]->getEc();
	
	
	next = NULL;

	for(i = 0; i < Nodes[pos]->succNumber; i++){
		if((Nodes[pos]->succs[i]->succNumber < 0) || (Nodes[pos]->succs[i]->succs[0]->getId() != leaf)){
			cout << "Error in the specified leaf" << endl;
			if (Nodes[pos]->succs[i]->succs[0]->getId() != leaf)
			cout << "(root, leaf) = " << root << ", "  << Nodes[pos]->succs[i]->succs[0]->getId() << endl;
			return;
		}else{
		
			// QoS computation 		
			srt = srt + Nodes[pos]->prbSucc[i]*Nodes[pos]->succs[i]->getSrt();
			ec = ec + Nodes[pos]->prbSucc[i]*Nodes[pos]->succs[i]->getEc();

			if(next == NULL)
				next = Nodes[pos]->succs[0]->succs[0];
			// We prepare and delete the next node
			Nodes[pos]->succs[i]->removeallSuccessors();
			Nodes[pos]->succs[i]->removeallPredecessors();
			Nodes[pos]->succs[i] = NULL;
		}
	}


	// Link the And Split with the Join successor

	
	// The problem are in the removal of predecessors and successors. Fix it

	if(next->succNumber > 0){
		Nodes[pos]->succNumber = 1;
		Nodes[pos]->succs[0] = next->succs[0];
		next->succs[0]->removePredecessor(next);
		next->removeallSuccessors();
		next->removeallPredecessors();
	}else{
		Nodes[pos]->succNumber = 0;
		Nodes[pos]->succs[0] = NULL;
	}
	Nodes[pos]->setSrt(srt);
	Nodes[pos]->setEc(ec);

}


void OperationGraph :: or_reduction(int root, int leaf){

	BPMNUnitTask *next;	
	int i, pos;
	float srt, ec;	

	// We find the root pointer and initialize srt and ec	
	pos = getRootNode(root);
	srt = Nodes[pos]->getSrt();
	ec = Nodes[pos]->getEc();
	
	if(Nodes[pos]->succNumber != 2)
		cout << "error in the Or construct " << endl;

	for(i = 0; i < Nodes[pos]->succNumber; i++){
		if((Nodes[pos]->succs[i]->succNumber < 0) || (Nodes[pos]->succs[i]->succs[0]->getId() != leaf)){
			cout << "Error in the specified leaf" << endl;
			cout << "(root, leaf) = " << root << ", "  << leaf << endl;
			return;
		}
	}

	srt = srt + Nodes[pos]->prbSucc[0]*Nodes[pos]->succs[0]->getSrt()+ Nodes[pos]->prbSucc[1]*Nodes[pos]->succs[1]->getSrt()+ Nodes[pos]->prbSucc[2]*max(Nodes[pos]->succs[0]->getSrt(), Nodes[pos]->succs[1]->getSrt());
	ec = ec + Nodes[pos]->prbSucc[0]*Nodes[pos]->succs[0]->getEc()+ Nodes[pos]->prbSucc[1]*Nodes[pos]->succs[1]->getEc()+ Nodes[pos]->prbSucc[2]*(Nodes[pos]->succs[0]->getEc()+Nodes[pos]->succs[1]->getEc());
		

	next = Nodes[pos]->succs[0]->succs[0];
	// We prepare and delete the next node
	Nodes[pos]->succs[0]->removeallSuccessors();
	Nodes[pos]->succs[0]->removeallPredecessors();
	Nodes[pos]->succs[0] = NULL;

	Nodes[pos]->succs[1]->removeallSuccessors();
	Nodes[pos]->succs[1]->removeallPredecessors();
	Nodes[pos]->succs[1] = NULL;

	// Link the And Split with the Join successor

	if(next->succNumber > 0){
		Nodes[pos]->succNumber = 1;
		Nodes[pos]->succs[0] = next->succs[0];
		next->succs[0]->removePredecessor(next);
		next->removeallSuccessors();
		next->removeallPredecessors();
	}else{
		Nodes[pos]->succNumber = 0;
		Nodes[pos]->succs[0] = NULL;
	}
		Nodes[pos]->setSrt(srt);
		Nodes[pos]->setEc(ec);
		
}


void OperationGraph :: loop_reduction(){
	int i;
	for(i = 0; i < size; i++){
		if(Nodes[i]->nloop > 0){
			Nodes[i]->setSrt(Nodes[i]->getSrt()*Nodes[i]->nloop);
			Nodes[i]->setEc(Nodes[i]->getEc()*Nodes[i]->nloop);
			Nodes[i]->predLoop = NULL;
		}
	}
}


void OperationGraph :: reduce_with_order(){

	int pos;
	int i;
	
	graphAva = 1;
	graphRel = 1;
	for (i = 0; i < size; i++) {
		if (isOperation(Nodes[i])){
			graphAva = graphAva * Nodes[i]->getAva();
			graphRel = graphRel * Nodes[i]->getRel();
			
		}
	}
	
	loop_reduction();
	computeOrder();
	for(i = 0; i < ordersize; i++){	

//		cout << "order = "<< Order[i].root << ", " << Order[i].leaf << endl;
		
		pos = getRootNode(Order[i].root);

		if(Nodes[pos]->succNumber == 1){
			sequence_reduction(Order[i].root);
		}
		else{
			switch(Order[i].type_root){
				case XOR_SPLIT: 
					xor_reduction(Order[i].root, Order[i].leaf);
					break;
				case AND_SPLIT:
					fork_reduction(Order[i].root, Order[i].leaf);
					break;
				case OR_SPLIT:
					or_reduction(Order[i].root, Order[i].leaf);
					break;
				default:
					cout << "error in the reduction order: type \n";
					break; 
			}
		} 


	}
	graphSrt = Nodes[0]->getSrt();
	graphEc = Nodes[0]->getEc();	



}


void OperationGraph :: reduce(int root){

	int pos;

	pos = getRootNode(root);

	if(Nodes[pos]->succNumber == 1)
			sequence_reduction(root);
	else{

		switch(Nodes[pos]->getType()){
			case XOR_SPLIT: 
				xor_reduction(Nodes[pos]->getId(), Nodes[pos]->succs[0]->succs[0]->getId());
				break;
			case AND_SPLIT:
				fork_reduction(Nodes[pos]->getId(), Nodes[pos]->succs[0]->succs[0]->getId());
				break;
			case OR_SPLIT:
				or_reduction(Nodes[pos]->getId(), Nodes[pos]->succs[0]->succs[0]->getId());
				break;
			default:
				cout << "error in the reduction order: type \n";
				break; 
		
			}
	}
	
}



int OperationGraph :: isReducible(int root){
	
	int pos, i;
	pos = getRootNode(root);
	int canReduce;	
	
	canReduce = FALSE;

	if(isClosed(Nodes[pos]))
		canReduce = FALSE;
	else{
		if(Nodes[pos]->succNumber == 0)	
			canReduce = FALSE;
		else{
			if(Nodes[pos]->getType() == OPERATION){
				if((Nodes[pos]->succs[0]->succNumber <= 1) && !isClosed(Nodes[pos]->succs[0]) )
					canReduce =  TRUE;
			}
			else{
				if(isOpen(Nodes[pos])){		
					if ((Nodes[pos]->succNumber == 1) && (!isClosed(Nodes[pos]->succs[0])) ) {
						canReduce = TRUE;
					}else{
					canReduce = TRUE;
					i = 0;
					while((i < Nodes[pos]->succNumber) && (canReduce == TRUE)){
						if((Nodes[pos]->succs[i]->succNumber == 1) && isClosed(Nodes[pos]->succs[i]->succs[0]) )
							i++;
						else
							canReduce = FALSE;
					}
					}
				}
			}
		}
	}
		
	return canReduce;
}




void OperationGraph :: reduce_rand_order(){

	int i, j, temp;
	int *tabOrd;

	loop_reduction();

	graphAva = 1;
	graphRel = 1;
	for (i = 0; i < size; i++) {
		if (isOperation(Nodes[i])){
			graphAva = graphAva * Nodes[i]->getAva();
			graphRel = graphRel * Nodes[i]->getRel();

		}
	}
	
	tabOrd = new int[size];
	for(i = 0; i < size; i++)
		tabOrd[i] = i;

	for(i = 1; i < size; i++){
		j = rand()%(size-2) + 1;
		if (j < 0) {
			j = (-1)*j;
		}
		temp = tabOrd[i];
		tabOrd[i] = tabOrd[j];
		tabOrd[j] = temp;
	}
/*	cout << "Random order" << endl;
	for(i = 0; i < size; i ++)
		cout << tabOrd[i] << endl;
*/	
	while(Nodes[0]->succNumber > 0){
		for(i = 0; i < size; i++){
			if(isReducible(Nodes[tabOrd[i]]->getId())){
//				cout << "id = " << Nodes[tabOrd[i]]->getId() << endl;
				reduce(Nodes[tabOrd[i]]->getId());
			}
		}
	}
	
	delete[] tabOrd;

	graphSrt = Nodes[0]->getSrt();
	graphEc = Nodes[0]->getEc();
	

}


void OperationGraph :: reduce_lex_order(){

	int i;
	loop_reduction();
	
	graphAva = 1;
	graphRel = 1;
	for (i = 0; i < size; i++) {
		if (isOperation(Nodes[i])){
			graphAva = graphAva * Nodes[i]->getAva();
			graphRel = graphRel * Nodes[i]->getRel();
			
		}
	}
	
	while(Nodes[0]->succNumber > 0){
		i = 0;
		for(i = 0; i < size; i++){
			if(isReducible(Nodes[i]->getId())){
//				cout << "id = " << Nodes[i]->getId() << endl;
				reduce(Nodes[i]->getId());
			}
		}
	}

	graphSrt = Nodes[0]->getSrt();
	graphEc = Nodes[0]->getEc();
}


float OperationGraph :: getSrt(){
	return graphSrt;
}


float OperationGraph :: getEc(){
	return graphEc;
}


float OperationGraph :: getAva(){
	return graphAva;
}


float OperationGraph :: getRel(){
	return graphRel;
}
