package it.univaq.di.choreos.CLTS;

import it.univaq.di.choreos.CLTS.impl.CLTSPackageImpl;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class CLTSMetamodelParser {
	static {
		CLTSPackageImpl.init();
	}

	public static Model parse(String modelFileName) {
		URI fileURI = URI.createFileURI(modelFileName);
		Resource resource = new XMIResourceFactoryImpl().createResource(fileURI);
		try {
			resource.load(null);
			Model clts = (Model) resource.getContents().get(0);
			return clts;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static Model getCltsModel(String cltsContent) {
		Model clts = null;
		String tempFolder = createTempFolder();
		try {
			File cltsFile = new File(tempFolder + File.separator + "cltstemp.clts");
			FileUtils.writeStringToFile(cltsFile, cltsContent);
			clts = parse(cltsFile.getAbsolutePath());
			FileUtils.deleteQuietly(new File(tempFolder));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return clts;
	}

	public static String getCltsContent(Model cltsModel) {
		String tempFolder = createTempFolder();
		File tempClts = new File(tempFolder + File.separator + "cltstemp.clts");
		// create resource from the Model
		URI fileUriTempModelNormalized = URI.createFileURI(tempClts.getAbsolutePath());
		Resource resourceModelNormalized = new XMIResourceFactoryImpl().createResource(fileUriTempModelNormalized);
		// add model in model resourceModel
		resourceModelNormalized.getContents().add(cltsModel);
		String cltsContent = "";
		try {
			// save new CLTS model
			resourceModelNormalized.save(Collections.EMPTY_MAP);
			cltsContent = FileUtils.readFileToString(tempClts);
			FileUtils.deleteQuietly(new File(tempFolder));

		} catch (IOException e) {
			e.printStackTrace();
		}
		return cltsContent;
	}
	
	private static String createTempFolder(){
		String tempFolder = getTempFolder();
		File tempFolderDir = new File(tempFolder);
		try {
			FileUtils.forceMkdir(tempFolderDir);
			return tempFolder;
		} catch (IOException e) {
			e.printStackTrace();
		}finally {
			try {
				FileUtils.deleteDirectory(tempFolderDir);
			} catch (IOException e) {
			}
		}
		return "";
	}

	private static String getTempFolder() {
		return FileUtils.getTempDirectoryPath() + File.separator + getRandomDir() + File.separator;
	}

	private static String getRandomDir() {
		return new Date().getTime() + "";
	}



}