package it.univaq.di.choreos.elts.strongsimulation;

import it.univaq.di.choreos.ELTS.ActivityLoop;
import it.univaq.di.choreos.ELTS.ActivityLoop2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2ActivityLoop;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSManagerImpl;

import java.util.List;

/**
 * 
 * @author Alexander Perucci
 *
 */
public class RemoveActivityLoopObjects {
	private ELTSModel modelManager;
	
	public RemoveActivityLoopObjects(ELTSModel modelManager){
		this.modelManager=modelManager;
		removeActivityLoopObjects();
	}
	
	/**
	 * @return the model without ActivityLoops
	 */
	public Model getModelWithoutActivityLoopObjetcs() {
		return modelManager.getModel();
	}
	
	/**
	 * Remove ActivityLoops
	 */
	private void removeActivityLoopObjects(){
		for (ActivityLoop activityLoop : modelManager.getActivityLoops()) {
			//all transitions to activityLoop
			List<EpsilonTransition> epsilonTransitionsToActivityLoop =modelManager.getAllTransitionsToActivityLoop(activityLoop);
			//all transitions from activityLoop
			List<EpsilonTransition> epsilonTransitionsFromActivityLoop =modelManager.getAllTransitionsFromActivityLoop(activityLoop);	
			//remove activity loop
			removeActivityLoopToModel(activityLoop, epsilonTransitionsToActivityLoop, epsilonTransitionsFromActivityLoop);

		}
		//remove all activity loops to the model
		while (modelManager.getActivityLoops().size()!=0){
			modelManager.removeActivityLoop(modelManager.getActivityLoops().get(0));
		}
		
	}
	

	/**
	 * 
	 * @param activityLoop activity loop to remove
	 * @param epsilonTransitionsToActivityLoop all transitions to activityLoop
	 * @param epsilonTransitionsFromActivityLoop all transitions from activityLoop
	 */
	private void removeActivityLoopToModel(ActivityLoop activityLoop, List<EpsilonTransition> epsilonTransitionsToActivityLoop,List<EpsilonTransition> epsilonTransitionsFromActivityLoop ){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
		// create a new state
		State newState = cltsFactoryManager.createState(activityLoop.getName());
		modelManager.insertState(newState);

		// add transitions 
		for (EpsilonTransition epsilonTransition : epsilonTransitionsFromActivityLoop) {
			if (epsilonTransition instanceof ActivityLoop2State){
				if (((ActivityLoop2State) epsilonTransition).getSource().getName().compareTo(activityLoop.getName())==0){
					State2State ess =cltsFactoryManager.createState2State(newState, ((ActivityLoop2State) epsilonTransition).getTarget());
					modelManager.insertState2State(ess);
				}
			}
		}
		//remove all transition from activity loop
		modelManager.removeEpsilonTransitions(epsilonTransitionsFromActivityLoop);

		// add transitions
		for (EpsilonTransition epsilonTransition : epsilonTransitionsToActivityLoop) {
			if (epsilonTransition instanceof State2ActivityLoop){
				if (((State2ActivityLoop) epsilonTransition).getTarget().getName().compareTo(activityLoop.getName())==0){
					State2State ess=cltsFactoryManager.createState2State(((State2ActivityLoop) epsilonTransition).getSource(), newState);
					modelManager.insertState2State(ess);
				}
			}
		}
		//remove all transition to activity loop
		modelManager.removeEpsilonTransitions(epsilonTransitionsToActivityLoop);
	}



}