package it.univaq.di.choreos.elts.strongsimulation;

import it.univaq.di.choreos.ELTS.Alternative;
import it.univaq.di.choreos.ELTS.Alternative2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2Alternative;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSManagerImpl;

import java.util.List;

/**
 * 
 * @author Alexander Perucci
 *
 */
public class RemoveAlternativeObjects {
	private ELTSModel modelManager;
	
	public RemoveAlternativeObjects(ELTSModel modelManager){
		this.modelManager = modelManager;
		removeAlternativeObjects();
	}

	/**
	 * 
	 * @return the model without Alternatives 
	 */
	public Model getModelWithoutAlternativeObjetcs() {
		return modelManager.getModel();
	}
	
	/**
	 * Remove Alternatives
	 */
	public void removeAlternativeObjects(){
		for (Alternative alternative : modelManager.getAlternatives()) {
			//all transitions to alternative
			List<EpsilonTransition> epsilonTransitionsToAlternative =modelManager.getAllTransitionsToAlternative(alternative);
			//all transitions from alternative
			List<EpsilonTransition> epsilonTransitionsFromAlternative =modelManager.getAllTransitionsFromAlternative(alternative);	
			//remove activity loop
			removeAlternativeToModel(alternative, epsilonTransitionsToAlternative, epsilonTransitionsFromAlternative);
		}
		//remove all alternative to the model
		while (modelManager.getAlternatives().size()!=0){
			modelManager.removeAlternative(modelManager.getAlternatives().get(0));
		}
	}
	

	/**
	 * 
	 * @param alternative alternative to remove
	 * @param epsilonTransitionsToAlternative all transitions to alternative
	 * @param epsilonTransitionsFromAlternative all transitions from alternative
	 */
	private void removeAlternativeToModel(Alternative alternative, List<EpsilonTransition> epsilonTransitionsToAlternative,List<EpsilonTransition> epsilonTransitionsFromAlternative ){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
		// create a new state
		State newState = cltsFactoryManager.createState(alternative.getName());
		modelManager.insertState(newState);

		//  add transitions 
		for (EpsilonTransition epsilonTransition : epsilonTransitionsFromAlternative) {
			if (epsilonTransition instanceof Alternative2State){
				if (((Alternative2State) epsilonTransition).getSource().getName().compareTo(alternative.getName())==0){
					State2State ess=cltsFactoryManager.createState2State(newState, ((Alternative2State) epsilonTransition).getTarget());
					modelManager.insertState2State(ess);
				}
			}
		}

		//remove all transition from alternative
		modelManager.removeEpsilonTransitions(epsilonTransitionsFromAlternative);

		// add transitions
		for (EpsilonTransition epsilonTransition : epsilonTransitionsToAlternative) {
			if (epsilonTransition instanceof State2Alternative){
				if (((State2Alternative) epsilonTransition).getTarget().getName().compareTo(alternative.getName())==0){
					State2State ess =cltsFactoryManager.createState2State(((State2Alternative) epsilonTransition).getSource(), newState);
					modelManager.insertState2State(ess);
				}
			}
		}
		//remove all transition to alternative
		modelManager.removeEpsilonTransitions(epsilonTransitionsToAlternative);
	}
}
