package it.univaq.di.choreos.elts.strongsimulation;

import it.univaq.di.choreos.ELTS.ActivityLoop2State;
import it.univaq.di.choreos.ELTS.Alternative2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.Fork2State;
import it.univaq.di.choreos.ELTS.InitialState2State;
import it.univaq.di.choreos.ELTS.Join2State;
import it.univaq.di.choreos.ELTS.LabeledState2State;
import it.univaq.di.choreos.ELTS.LabeledTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2ActivityLoop;
import it.univaq.di.choreos.ELTS.State2Alternative;
import it.univaq.di.choreos.ELTS.State2FinalState;
import it.univaq.di.choreos.ELTS.State2Fork;
import it.univaq.di.choreos.ELTS.State2Join;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.ELTS.Transition;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSManagerImpl;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author Alexander Perucci
 *
 */
public class RemoveEpsilonState2StateObjects {
	private ELTSModel modelManager;
	
	public RemoveEpsilonState2StateObjects(ELTSModel modelManager){
		this.modelManager=modelManager;
		
		removeEpsilonState2StateObjects();
	}
	
	/**
	 * 
	 * @return the model without Epsilon State 2 State
	 */
	public Model getModelWithoutEpsilonState2StateObjetcs() {
		return modelManager.getModel();
	}
	
	/**
	 * Remove Epsilon State2State
	 */
	private void removeEpsilonState2StateObjects(){
		
		if ((modelManager.getAllEpsilonState2StateTransitions()).size()!=0){
			//remove all epsilon state 2 state with state source and state target are equals
			modelManager.removeState2StateWithSourceAndTargetEquals() ;
			// remove isolate state
			modelManager.removeStateAndTransitionIsolated();
			
			
			List<State2State> state2States=modelManager.getAllEpsilonState2StateTransitions();

			int element=0;
			while(state2States.size()!=0 && state2States.size()!=element){
				State2State state2StateToRemove = state2States.get(element);
				if (removeEpsilonState2StateToModel (state2StateToRemove)){
					modelManager.removeState2State(state2StateToRemove);
					//remove all epsilon state 2 state with state source and state target are equals
					modelManager.removeState2StateWithSourceAndTargetEquals() ;
					// remove isolate state
					modelManager.removeStateAndTransitionIsolated();
					state2States=modelManager.getAllEpsilonState2StateTransitions();
					element = 0;
				}else{
					element++;
				}
			}
		}
	}
	
	/**
	 * 
	 * @param state2State state2state transition to remove
	 */
	private boolean removeEpsilonState2StateToModel ( State2State state2State){
		State source = state2State.getSource();
		State target = state2State.getTarget();
		
		//get all transitions to state target
		List <Transition> transitionsToStateTarget = modelManager.getAllTransitionsToState(target);
		transitionsToStateTarget.remove(state2State);
		// get all transition from state source
		List <Transition> transitionsFromStateSource = modelManager.getAllTransitionsFromState(source);
		transitionsFromStateSource.remove(state2State);

		// state source has not transition From or state target has not transition To
		if (transitionsFromStateSource.size()==0 || transitionsToStateTarget.size()==0) {
			collapseStates(source, modelManager.getAllTransitionsFromState(target), transitionsToStateTarget);
			//remove state
			modelManager.removeState(target);
			return true;
		}
		return false;
		
	}
	
	/**
	 * 
	 * @param newState new state for transition
	 * @param transitionsFromStateTarget transitions to move
	 * @param transitionsToStateTarget transitions to move
	 */
	private void collapseStates(State newState, List <Transition> transitionsFromStateTarget, List <Transition> transitionsToStateTarget){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
	
		// move transitions from state if transition not already exits
		List<Transition> transitionToRemoved=new ArrayList<Transition>();
		for (Transition transition : transitionsFromStateTarget) {
			if (transition instanceof State2State){
				State2State ss=cltsFactoryManager.createState2State(newState, ((State2State)transition).getTarget());
				if (!modelManager.existTransition(ss))
					modelManager.editSourceState2State((State2State) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof State2FinalState){
				State2FinalState sf =cltsFactoryManager.createState2FinalState(newState, ((State2FinalState) transition).getTarget());
				if (!modelManager.existTransition(sf))
					modelManager.editSourceState2FinalState((State2FinalState) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof State2Fork){
				State2Fork sf =cltsFactoryManager.createState2Fork(newState, ((State2Fork) transition).getTarget());
				if (!modelManager.existTransition(sf))
					modelManager.editSourceState2Fork((State2Fork) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
			
			else if (transition instanceof State2Join){
				State2Join sj =cltsFactoryManager.createState2Join(newState, ((State2Join) transition).getTarget());
				if (!modelManager.existTransition(sj))
					modelManager.editSourceState2Join((State2Join) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof State2ActivityLoop){
				State2ActivityLoop al =cltsFactoryManager.createState2ActivityLoop(newState, ((State2ActivityLoop) transition).getTarget());
				if (!modelManager.existTransition(al))
					modelManager.editSourceState2ActivityLoop((State2ActivityLoop) transition, newState);
				else
					transitionToRemoved.add(transition);
			}else if (transition instanceof State2Alternative){
				State2Alternative sa =cltsFactoryManager.createState2Alternative(newState, ((State2Alternative) transition).getTarget());
				if (!modelManager.existTransition(sa))
					modelManager.editSourceState2Alternative((State2Alternative) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof LabeledState2State){
				LabeledState2State lss= cltsFactoryManager.createLabeledState2State(newState, ((LabeledState2State) transition).getTarget(), ((LabeledState2State) transition).getOperationType() , ((LabeledState2State) transition).getLabel());
				if (!modelManager.existTransition(lss))
					modelManager.editSourceLabeledState2State((LabeledState2State) transition, newState);
				else
					transitionToRemoved.add(transition);
			}
		}
		
		// move transitions to state if transition not already exits
		for (Transition transition : transitionsToStateTarget) {
			if (transition instanceof State2State){
				State2State ss=cltsFactoryManager.createState2State(((State2State) transition).getSource(), newState);
				if (!modelManager.existTransition(ss))
					((State2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof InitialState2State){
				InitialState2State is =cltsFactoryManager.createInitialState2State(((InitialState2State) transition).getSource(), newState);
				if (!modelManager.existTransition(is))
					((InitialState2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof Fork2State){
				Fork2State fs =cltsFactoryManager.createFork2State(((Fork2State) transition).getSource(), newState);
				if (!modelManager.existTransition(fs))
					((Fork2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}else if (transition instanceof Join2State){
				Join2State js =cltsFactoryManager.createJoin2State(((Join2State) transition).getSource(), newState);
				if (!modelManager.existTransition(js))
					((Join2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}else if (transition instanceof ActivityLoop2State){
				ActivityLoop2State ls =cltsFactoryManager.createActivityLoop2State(((ActivityLoop2State) transition).getSource(), newState);
				if (!modelManager.existTransition(ls))
					((ActivityLoop2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}else if (transition instanceof Alternative2State){
				Alternative2State as =cltsFactoryManager.createAlternative2State(((Alternative2State) transition).getSource(), newState);
				if (!modelManager.existTransition(as))
					((Alternative2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}
			else if (transition instanceof LabeledState2State){
				LabeledState2State lss= cltsFactoryManager.createLabeledState2State(((LabeledState2State) transition).getSource(), newState,((LabeledState2State) transition).getOperationType(), ((LabeledState2State) transition).getLabel());
				if (!modelManager.existTransition(lss))
					((LabeledState2State) transition).setTarget(newState);
				else
					transitionToRemoved.add(transition);
			}
		}
		
		//remove transition not added
		for (Transition transition : transitionToRemoved) {
			if (transition instanceof LabeledTransition)
				modelManager.removeLabeledTransition((LabeledTransition)transition);
			else if (transition instanceof EpsilonTransition)
				modelManager.removeEpsilonTransition((EpsilonTransition)transition);
		}
		
	}
	
	
}
