package it.univaq.di.choreos.elts.strongsimulation.impl;

import it.univaq.di.choreos.ELTS.ActivityLoop;
import it.univaq.di.choreos.ELTS.ActivityLoop2State;
import it.univaq.di.choreos.ELTS.Alternative;
import it.univaq.di.choreos.ELTS.Alternative2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.FinalState;
import it.univaq.di.choreos.ELTS.Fork;
import it.univaq.di.choreos.ELTS.Fork2State;
import it.univaq.di.choreos.ELTS.InitialState;
import it.univaq.di.choreos.ELTS.InitialState2State;
import it.univaq.di.choreos.ELTS.Join;
import it.univaq.di.choreos.ELTS.Join2State;
import it.univaq.di.choreos.ELTS.LabeledState2State;
import it.univaq.di.choreos.ELTS.LabeledTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.NamedElement;
import it.univaq.di.choreos.ELTS.Operation;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2ActivityLoop;
import it.univaq.di.choreos.ELTS.State2Alternative;
import it.univaq.di.choreos.ELTS.State2FinalState;
import it.univaq.di.choreos.ELTS.State2Fork;
import it.univaq.di.choreos.ELTS.State2Join;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.ELTS.Transition;
import it.univaq.di.choreos.elts.strongsimulation.ELTSModel;
import it.univaq.di.choreos.elts.strongsimulation.NormalizeModel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * 
 * @author Alexander Perucci
 * 
 */
public class ELTSModelImpl implements ELTSModel {
	private Model model;
	
	public ELTSModelImpl(Model model){
		this.model=model;
	}
	
	@Override
	public Model getModel() {
		return this.model;
	}
	
	@Override
	public Model getModelNormalized() {
		NormalizeModel normalizeModel = new NormalizeModel(this);
		return normalizeModel.getModelNormalized();
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getOperation(it.univaq.di.choreos.CLTS.Model, java.lang.String)
	 */
	@Override
	public Operation getOperation(String operationName){
		for (Operation operation : model.getOperationsDefinition().getOperations()) {
			if(operation.getName().compareTo(operationName)==0)
				return operation;
		}
		return null;
	}
	
	
	/*
	 * get NamedElement
	 */
	@Override
	public List<ActivityLoop> getActivityLoops() {
		return model.getActivityLoops();
	}

	@Override
	public List<Alternative> getAlternatives() {
		return model.getAlternatives();
	}

	@Override
	public List<Fork> getForks() {
		return model.getForks();
	}

	@Override
	public List<Join> getJoins() {
		return model.getJoins();
	}

	@Override
	public List<State> getStates() {
		return model.getStates();
	}

	@Override
	public FinalState getFinalState() {
		return model.getFinalState();
	}

	@Override
	public InitialState getInitialState() {
		return model.getInitialState();
	}

	
	/*
	 * insert NamedElement
	 */
	
	@Override
	public void insertState(State state) {
		getStates().add(state);
	}

	@Override
	public void insertStates(List<State> states) {
		getStates().addAll(states);
	}
	
	@Override
	public void insertFork(Fork fork) {
		getForks().add(fork);
	}

	@Override
	public void insertJoin(Join join) {
		getJoins().add(join);
	}

	@Override
	public void insertAlternative(Alternative alternative) {
		getAlternatives().add(alternative);
	}

	@Override
	public void insertActivityLoop(ActivityLoop activityLoop) {
		getActivityLoops().add(activityLoop);
	}
	
	
	/*
	 * remove NamedElement
	 */
	
	@Override
	public void removeState(State state) {
		getStates().remove(state);
	}

	@Override
	public void removeFork(Fork fork) {
		getForks().remove(fork);
	}

	@Override
	public void removeJoin(Join join) {
		getJoins().remove(join);
	}

	@Override
	public void removeAlternative(Alternative alternative) {
		getAlternatives().remove(alternative);
	}

	@Override
	public void removeActivityLoop(ActivityLoop activityLoop) {
		getActivityLoops().remove(activityLoop);
	}

	@Override
	public void removeStates(List<State> states) {
		getStates().removeAll(states);
	}

	@Override
	public void removeForks(List<Fork> forks) {
		getForks().removeAll(forks);
	}

	@Override
	public void removeJoins(List<Join> joins) {
		getJoins().removeAll(joins);
	}

	@Override
	public void removeAlternatives(List<Alternative> alternatives) {
		getAlternatives().removeAll(alternatives);
	}

	@Override
	public void removeActivityLoops(List<ActivityLoop> activityLoops) {
		getActivityLoops().removeAll(activityLoops);
	}
	
	/*
	 * get Transition
	 */
	@Override
	public List<LabeledTransition> getLabeledTransitions() {
		return model.getLabeledTransitions();
	}

	@Override
	public List<EpsilonTransition> getEpsilonTransitions() {
		return model.getEpsilonTransitions();
	}

	

	
	
	/*
	 * insert Labeled Transition
	 */
	@Override
	public void insertLabeledTransition(LabeledTransition labeledTransition) {
		getLabeledTransitions().add(labeledTransition);		
	}

	@Override
	public void insertLabeledState2State(LabeledState2State labeledState2State) {
		getLabeledTransitions().add(labeledState2State);
	}

	
	/*
	 * insert Epsilon Transition
	 */
	@Override
	public void insertEpsilonTransition(EpsilonTransition epsilonTransition) {
		getEpsilonTransitions().add(epsilonTransition);
	}

	@Override
	public void insertJoin2State(Join2State join2State) {
		getEpsilonTransitions().add(join2State);
	}

	@Override
	public void insertState2Fork(State2Fork state2Fork) {
		getEpsilonTransitions().add(state2Fork);
	}

	@Override
	public void insertFork2State(Fork2State fork2State) {
		getEpsilonTransitions().add(fork2State);
	}

	

	@Override
	public void insertState2Join(State2Join state2Join) {
		getEpsilonTransitions().add(state2Join);
	}

	
	

	@Override
	public void insertInitialState2State(InitialState2State initialState2State) {
		getEpsilonTransitions().add(initialState2State);
	}

	@Override
	public void insertState2FinalState(State2FinalState state2FinalState) {
		getEpsilonTransitions().add(state2FinalState);
	}

	@Override
	public void insertState2Alternative(State2Alternative state2Alternative) {
		getEpsilonTransitions().add(state2Alternative);
	}

	@Override
	public void insertAlternative2State(Alternative2State alternative2State) {
		getEpsilonTransitions().add(alternative2State);
	}

	@Override
	public void insertState2ActivityLoop(State2ActivityLoop state2ActivityLoop) {
		getEpsilonTransitions().add(state2ActivityLoop);
	}

	@Override
	public void insertActivityLoop2State(ActivityLoop2State activityLoop2State) {
		getEpsilonTransitions().add(activityLoop2State);
	}

	
	@Override
	public void insertState2State(State2State state2State) {
		getEpsilonTransitions().add(state2State);
	}


	
	
	/*
	 * remove Labeled Transition
	 */
	@Override
	public void removeLabeledTransition(LabeledTransition labeledTransition) {
		getLabeledTransitions().remove(labeledTransition);		
	}
	
	@Override
	public void removeLabeledTransitions(List<LabeledTransition> labeledTransitions) {
		getLabeledTransitions().removeAll(labeledTransitions);		
	}
	
	@Override
	public void removeLabeledState2State(LabeledState2State labeledState2State) {
		getLabeledTransitions().remove(labeledState2State);
	}


	/*
	 * remove Epsilon Transition
	 */
	
	@Override
	public void removeEpsilonTransitions(List<EpsilonTransition> epsilonTransitions) {
		getEpsilonTransitions().removeAll(epsilonTransitions);
	}
	
	@Override
	public void removeEpsilonTransition(EpsilonTransition epsilonTransition){
		getEpsilonTransitions().remove(epsilonTransition);
	}
	@Override
	public void removeJoin2State(Join2State join2State) {
		getEpsilonTransitions().remove(join2State);
	}

	@Override
	public void removeState2Fork(State2Fork state2Fork) {
		getEpsilonTransitions().remove(state2Fork);
	}

	@Override
	public void removeFork2State(Fork2State fork2State) {
		getEpsilonTransitions().remove(fork2State);
	}

	

	@Override
	public void removeState2Join(State2Join state2Join) {
		getEpsilonTransitions().remove(state2Join);
	}

	

	@Override
	public void removeInitialState2State(InitialState2State initialState2State) {
		getEpsilonTransitions().remove(initialState2State);
	}

	@Override
	public void removeState2FinalState(State2FinalState state2FinalState) {
		getEpsilonTransitions().remove(state2FinalState);
	}

	@Override
	public void removeState2Alternative(State2Alternative state2Alternative) {
		getEpsilonTransitions().remove(state2Alternative);
	}

	@Override
	public void removeAlternative2State(Alternative2State alternative2State) {
		getEpsilonTransitions().remove(alternative2State);
	}

	@Override
	public void removeState2ActivityLoop(State2ActivityLoop state2ActivityLoop) {
		getEpsilonTransitions().remove(state2ActivityLoop);
	}

	@Override
	public void removeActivityLoop2State(ActivityLoop2State activityLoop2State) {
		getEpsilonTransitions().remove(activityLoop2State);
	}

	

	@Override
	public void removeState2State(State2State state2State) {
		getEpsilonTransitions().remove(state2State);
	}

	

	/*
	 * edit source transitions
	 */
	@Override
	public void editSourceLabeledState2State(LabeledState2State labeledState2State, State state) {
		labeledState2State.setSource(state);
	}

	

	@Override
	public void editSourceJoin2State(Join2State join2State, Join join) {
		join2State.setSource(join);
	}

	@Override
	public void editSourceState2Fork(State2Fork state2Fork, State state) {
		state2Fork.setSource(state);
	}

	@Override
	public void editSourceFork2State(Fork2State fork2State, Fork fork) {
		fork2State.setSource(fork);
	}

	

	@Override
	public void editSourceState2Join(State2Join state2Join, State state) {
		state2Join.setSource(state);
	}

	

	@Override
	public void editSourceInitialState2State(InitialState2State initialState2State, InitialState initialState) {
		initialState2State.setSource(initialState);
	}

	@Override
	public void editSourceState2FinalState(State2FinalState state2FinalState, State state) {
		state2FinalState.setSource(state);
	}

	@Override
	public void editSourceState2Alternative(State2Alternative state2Alternative, State state) {
		state2Alternative.setSource(state);
	}

	@Override
	public void editSourceAlternative2State(Alternative2State alternative2State, Alternative alternative) {
		alternative2State.setSource(alternative);
	}

	@Override
	public void editSourceState2ActivityLoop(State2ActivityLoop state2ActivityLoop, State state) {
		state2ActivityLoop.setSource(state);
	}

	@Override
	public void editSourceActivityLoop2State(ActivityLoop2State activityLoop2State, ActivityLoop activityLoop) {
		activityLoop2State.setSource(activityLoop);
	}

	

	

	@Override
	public void editSourceState2State(State2State state2State, State state) {
		state2State.setSource(state);
	}

	

	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllInitial2StateTransitions(it.univaq.di.choreos.CLTS.Model)
	 */
	@Override
	public  List<InitialState2State> getAllInitial2StateTransitions() {
		// create a list of transitions starting from the Initial State
		List<InitialState2State> transitionFromInitialState = new ArrayList<InitialState2State>();

		// get all Epsilon Transitions
		List<EpsilonTransition> listEpsilonTransition = getEpsilonTransitions();

		for (EpsilonTransition epsilonTransition : listEpsilonTransition) {
			if (epsilonTransition instanceof InitialState2State)
				if (((InitialState2State) epsilonTransition).getSource().equals(getInitialState()))
					transitionFromInitialState.add((InitialState2State) epsilonTransition);
		}

		return transitionFromInitialState;
	}

	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#existTransitionToFinalState(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public boolean existTransitionToFinalState( State state) {
		// get all Epsilon Transitions
		List<EpsilonTransition> listEpsilonTransition = getEpsilonTransitions();
		for (EpsilonTransition epsilonTransition : listEpsilonTransition) {
			if (epsilonTransition instanceof State2FinalState)
				if (((State2FinalState) epsilonTransition).getSource().equals(state))
					return true;
		}
		return false;

	}

	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllState2StateTransitionsFromStateSource(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public List<LabeledState2State> getAllState2StateTransitionsFromStateSource(State stateSource) {
		// create a list of State2StateTransition that contains the transitions from a Source State
		List<LabeledState2State> transitionFromState = new ArrayList<LabeledState2State>();
		// get all Labeled Transition
		List<LabeledTransition> allTransition = getLabeledTransitions();
		for (int i = 0; i < allTransition.size(); i++) {
			// if the transition selected is a State2StateTransition
			if (allTransition.get(i) instanceof LabeledState2State) {
				// get the Source State from the transition selected
				State state = ((LabeledState2State) allTransition.get(i)).getSource();
				// if the state selected has the same name of the source state
				// then insert the transition
				if (state.getName().compareTo(stateSource.getName()) == 0)
					transitionFromState.add((LabeledState2State) allTransition.get(i));
			}
		}
		return transitionFromState;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllEpsilonState2StateTransitionsFromStateSource(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public  List<State2State> getAllEpsilonState2StateTransitionsFromStateSource(State stateSource){
		List<State2State> transitionFromState = new ArrayList<State2State>();
		for (EpsilonTransition epsilonState2State : getEpsilonTransitions()) {
			// if the transition selected is a EpsilonState2State
			if (epsilonState2State instanceof State2State) {
				// get the Source State from the transition selected
				State state = ((State2State) epsilonState2State).getSource();
				// if the state selected has the same name of the source state
				// then insert the transition
				if (state.getName().compareTo(stateSource.getName()) == 0)
					transitionFromState.add((State2State) epsilonState2State);
			}
		}
		return transitionFromState;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllEpsilonState2StateTransitions(it.univaq.di.choreos.CLTS.Model)
	 */
	@Override
	public List<State2State> getAllEpsilonState2StateTransitions(){
		List<State2State> state2States =new ArrayList<State2State>();
		//get all epsilon state 2 state 
		for (EpsilonTransition epTransition :getEpsilonTransitions()) {
			if (epTransition instanceof State2State){
				state2States.add((State2State)epTransition);
			}
		}
		return state2States;
	}
	
	
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllState2StateTransitionsAllowable(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.LabeledState2State, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public List<LabeledState2State> getAllState2StateTransitionsAllowable( LabeledState2State transition, State state) {
		// create a list of allowable transitions (they have the same label)
		List<LabeledState2State> transitionFromState = new ArrayList<LabeledState2State>();
		//check if exist State2State transitions (Epsilon)  from state
		if ((getAllEpsilonState2StateTransitionsFromStateSource(state)).size()==0){
			//get all state2State transition allowable from State2StateTransition (Epsilon)
			getAllLabeledState2StateTransitionsAllowable(transitionFromState, transition, state);
		}
		else{
			//get all state2State transition allowable from State2StateTransition (Epsilon)
			getAllEpsilonState2StateTransitionsAllowable(new ArrayList<State>(), transitionFromState, transition, state);
		}
	
		return transitionFromState;

	}
	
	/**
	 * 
	 * @param model model
	 * @param transitionFromState transitions allowable
	 * @param transition transition to check
	 * @param state state to start check
	 */
	private void getAllLabeledState2StateTransitionsAllowable (List<LabeledState2State> transitionFromState, LabeledState2State transitionToCheck, State state){
		// check if the transition exists in the State2State transition (Labeled)
		for (LabeledState2State state2StateTransition : getAllState2StateTransitionsFromStateSource(state)) {
			if (state2StateTransition.getLabel().getName().compareTo(transitionToCheck.getLabel().getName()) == 0)
				transitionFromState.add(state2StateTransition);
		}
		
		return;		
	}
	
	/**
	 * 
	 * @param model model
	 * @param statesAnalyzed list of states useful for end recursion
	 * @param transitionFromState transitions allowable
	 * @param transitionToCheck transition to check
	 * @param state
	 */
	private void getAllEpsilonState2StateTransitionsAllowable ( List<State> statesAnalyzed, List<LabeledState2State> transitionFromState, LabeledState2State transitionToCheck, State state){
		// check if state is already analyzed
		if (!statesAnalyzed.contains(state)){
			statesAnalyzed.add(state);	
		}
		else
			return;
		
		//get all labeled state2State transition allowable from State2StateTransition (Epsilon)
		getAllLabeledState2StateTransitionsAllowable(transitionFromState, transitionToCheck, state);
		
		// get all State2State (epsilon) transitions
		for (State2State epsilonState2State : getAllEpsilonState2StateTransitionsFromStateSource(state)) {
			// navigation model for each transition State2State
			getAllEpsilonState2StateTransitionsAllowable(statesAnalyzed,transitionFromState, transitionToCheck, epsilonState2State.getTarget());
			
		}
					
		return;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllForksFromState(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public List<Fork> getAllForksFromState (State state){
		List <EpsilonTransition> allEpsilonTransitions = getEpsilonTransitions();
		List<Fork> allForks = new ArrayList<Fork>();
		for (EpsilonTransition epsilonTransition: allEpsilonTransitions){
			if (epsilonTransition instanceof State2Fork){
				State stateSource= ((State2Fork)epsilonTransition).getSource();
				if (state.getName().compareTo(stateSource.getName())==0)
					allForks.add(((State2Fork) epsilonTransition).getTarget());
			}
			
			
		}
		return allForks;
		
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllStartStatesByFork(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Fork)
	 */
	@Override
	public List<State> getAllStartStatesByFork (Fork fork){
		List <EpsilonTransition> allEpsilonTransitions = getEpsilonTransitions();
		List<State> allStates = new ArrayList<State>();
		for (EpsilonTransition epsilonTransition: allEpsilonTransitions){
			if (epsilonTransition instanceof Fork2State){
				Fork forkSource= ((Fork2State)epsilonTransition).getSource();
				if (forkSource.getName().compareTo(fork.getName())==0){
					allStates.add(((Fork2State) epsilonTransition).getTarget());
				}
			}
		}
		return allStates;
		
	}

	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getJoinConnectedByState(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */

 	@Override
	public Join getJoinConnectedByState (State state){
		List <EpsilonTransition> allEpsilonTransitions = getEpsilonTransitions();
		for (EpsilonTransition epsilonTransition: allEpsilonTransitions){
			if (epsilonTransition instanceof State2Join){
				State stateSource=((State2Join) epsilonTransition).getSource();
				if (stateSource.getName().compareTo(state.getName())==0)
					return ((State2Join) epsilonTransition).getTarget();
			}
			
		}
		return null;
	}

	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getNamedElementConnectedByJoin(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Join)
	 */
	@Override
	public NamedElement getNamedElementConnectedByJoin (Join join){
		
		List<EpsilonTransition> allEpsilonTransitions = getEpsilonTransitions();
		for (EpsilonTransition epsilonTransition : allEpsilonTransitions) {
			if (epsilonTransition instanceof Join2State) {
				Join joinSource = ((Join2State) epsilonTransition).getSource();
				if (joinSource.getName().compareTo(join.getName()) == 0)
					return ((Join2State) epsilonTransition).getTarget();
			} 
		}
		
		return null;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getTransitionFromJoin(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Join)
	 */
	@Override
	public EpsilonTransition getTransitionFromJoin ( Join join){
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if  (epsilonTransition instanceof Join2State){
				if (((Join2State) epsilonTransition).getSource().getName().compareTo(join.getName())==0)
					return epsilonTransition;
				
			}
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getNamedElementConnectedToFork(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Fork)
	 */
	@Override
	public NamedElement getNamedElementConnectedToFork (Fork fork){

		List<EpsilonTransition> allEpsilonTransitions = getEpsilonTransitions();
		for (EpsilonTransition epsilonTransition : allEpsilonTransitions) {
			if (epsilonTransition instanceof State2Fork) {
				Fork forkTarget = ((State2Fork) epsilonTransition).getTarget();
				if (forkTarget.getName().compareTo(fork.getName()) == 0)
					return ((State2Fork) epsilonTransition).getSource();
			} 
		}
		
		return null;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getTransitionToFork(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Fork)
	 */
	@Override
	public EpsilonTransition getTransitionToFork ( Fork fork){
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if  (epsilonTransition instanceof State2Fork){
				if (((State2Fork) epsilonTransition).getTarget().getName().compareTo(fork.getName())==0)
					return epsilonTransition;
				
			}
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsFromFork(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Fork)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsFromFork (Fork fork){
		List<EpsilonTransition> epsilonTransitions=new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if  (epsilonTransition instanceof Fork2State){
				if (((Fork2State) epsilonTransition).getSource().getName().compareTo(fork.getName())==0)
					epsilonTransitions.add(epsilonTransition);
				
			}
			
		}
		return epsilonTransitions;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsToJoin(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Join)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsToJoin (Join join){
		List<EpsilonTransition> epsilonTransitions=new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if  (epsilonTransition instanceof State2Join){
				if (((State2Join) epsilonTransition).getTarget().getName().compareTo(join.getName())==0)
					epsilonTransitions.add(epsilonTransition);
				
			}
			
		}
		return epsilonTransitions;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsToState(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public List<Transition> getAllTransitionsToState ( State state){
		List <Transition> allTransition = new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
			if (epsilonTransition instanceof State2State){
				if (((State2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			} else if (epsilonTransition instanceof InitialState2State) {
				if (((InitialState2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			} else if (epsilonTransition instanceof Fork2State) {
				if (((Fork2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			} else if (epsilonTransition instanceof Join2State) {
				if (((Join2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			} else if (epsilonTransition instanceof ActivityLoop2State) {
				if (((ActivityLoop2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			} else if (epsilonTransition instanceof Alternative2State) {
				if (((Alternative2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0) 
					allTransition.add(epsilonTransition);
			} 
			
		}
		for (LabeledTransition labeledTransition : getLabeledTransitions()){
			if (labeledTransition instanceof LabeledState2State)
				if (((LabeledState2State) labeledTransition).getTarget().getName().compareTo(state.getName())==0)
					allTransition.add(labeledTransition);
		}
		
		return allTransition;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsFromState(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public List<Transition> getAllTransitionsFromState ( State state){
		List <Transition> allTransition = new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
			if (epsilonTransition instanceof State2State){
				if (((State2State) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2FinalState){
				if (((State2FinalState) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2Fork){
				if (((State2Fork) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2Join){
				if (((State2Join) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}
			else if (epsilonTransition instanceof State2ActivityLoop){
				if (((State2ActivityLoop) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}
			else if (epsilonTransition instanceof State2Alternative){
				if (((State2Alternative) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(epsilonTransition);
			}
		}
		for (LabeledTransition labeledTransition : getLabeledTransitions()){
			if (labeledTransition instanceof LabeledState2State){
				if (((LabeledState2State) labeledTransition).getSource().getName().compareTo(state.getName())==0)
					allTransition.add(labeledTransition);
			}
		}

		return allTransition;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getState2JoinTransition(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State, it.univaq.di.choreos.CLTS.Join)
	 */
	@Override
	public State2Join getState2JoinTransition( State state, Join join){
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof State2Join){
				if (((State2Join) epsilonTransition).getSource().getName().compareTo(state.getName())==0 &&
						((State2Join) epsilonTransition).getTarget().getName().compareTo(join.getName())==0){
					return (State2Join)epsilonTransition;
				}
			}
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getState2Final(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public State2FinalState getState2Final ( State state){
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof State2FinalState){
				if (((State2FinalState) epsilonTransition).getSource().getName().compareTo(state.getName())==0 &&
						((State2FinalState) epsilonTransition).getTarget().getName().compareTo(getFinalState().getName())==0){
					return (State2FinalState)epsilonTransition;
				}
			}
		}
		return null;
	}
	
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getFork2StateTransition(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Fork, it.univaq.di.choreos.CLTS.State)
	 */
	@Override
	public Fork2State getFork2StateTransition( Fork fork, State state){
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof Fork2State){
				if (((Fork2State) epsilonTransition).getSource().getName().compareTo(fork.getName())==0 &&
						((Fork2State) epsilonTransition).getTarget().getName().compareTo(state.getName())==0){
					return (Fork2State)epsilonTransition;
				}
			}
		}
		return null;
	}
	
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsToActivityLoop(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.ActivityLoop)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsToActivityLoop ( ActivityLoop activityLoop){
		List<EpsilonTransition> transitionsToActivityLoop =new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof State2ActivityLoop){
				if (((State2ActivityLoop) epsilonTransition).getTarget().getName().compareTo(activityLoop.getName())==0)
					transitionsToActivityLoop.add(epsilonTransition);
			}
		}
		
		return transitionsToActivityLoop;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsFromActivityLoop(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.ActivityLoop)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsFromActivityLoop ( ActivityLoop activityLoop){
		List<EpsilonTransition> transitionsFromActivityLoop =new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof ActivityLoop2State){
				if (((ActivityLoop2State) epsilonTransition).getSource().getName().compareTo(activityLoop.getName())==0)
					transitionsFromActivityLoop.add(epsilonTransition);
			}
		}
		
		
		
		return transitionsFromActivityLoop;
	}
	
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsToAlternative(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Alternative)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsToAlternative ( Alternative alternative){
		List<EpsilonTransition> transitionsToAlternative =new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof State2Alternative){
				if (((State2Alternative) epsilonTransition).getTarget().getName().compareTo(alternative.getName())==0)
					transitionsToAlternative.add(epsilonTransition);
			}
		}
		return transitionsToAlternative;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#getAllTransitionsFromAlternative(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Alternative)
	 */
	@Override
	public List<EpsilonTransition> getAllTransitionsFromAlternative ( Alternative alternative){
		List<EpsilonTransition> transitionsFromAlternative =new ArrayList<EpsilonTransition>();
		for (EpsilonTransition epsilonTransition : getEpsilonTransitions()) {
			if (epsilonTransition instanceof Alternative2State){
				if (((Alternative2State) epsilonTransition).getSource().getName().compareTo(alternative.getName())==0)
					transitionsFromAlternative.add(epsilonTransition);
			}
		}
		return transitionsFromAlternative;
	}
	
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#existTransition(it.univaq.di.choreos.CLTS.Model, it.univaq.di.choreos.CLTS.Transition)
	 */
	@Override
	public boolean existTransition(Transition transition){
		
		if(transition instanceof InitialState2State){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof InitialState2State &&
				   ((InitialState2State)epsilonTransition).getSource().getName().compareTo(((InitialState2State) transition).getSource().getName())==0 &&
				   ((InitialState2State)epsilonTransition).getTarget().getName().compareTo(((InitialState2State) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof State2FinalState){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof State2FinalState &&
				   ((State2FinalState) epsilonTransition).getSource().getName().compareTo(((State2FinalState) transition).getSource().getName())==0 &&
				   ((State2FinalState)epsilonTransition).getTarget().getName().compareTo(((State2FinalState) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof State2State){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof State2State &&
				   ((State2State) epsilonTransition).getSource().getName().compareTo(((State2State) transition).getSource().getName())==0 &&
				   ((State2State)epsilonTransition).getTarget().getName().compareTo(((State2State) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof Fork2State){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof Fork2State &&
				   ((Fork2State) epsilonTransition).getSource().getName().compareTo(((Fork2State) transition).getSource().getName())==0 &&
				   ((Fork2State)epsilonTransition).getTarget().getName().compareTo(((Fork2State) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof State2Fork){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof State2Fork &&
				   ((State2Fork) epsilonTransition).getSource().getName().compareTo(((State2Fork) transition).getSource().getName())==0 &&
				   ((State2Fork)epsilonTransition).getTarget().getName().compareTo(((State2Fork) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof Join2State){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof Join2State &&
				   ((Join2State) epsilonTransition).getSource().getName().compareTo(((Join2State) transition).getSource().getName())==0 &&
				   ((Join2State)epsilonTransition).getTarget().getName().compareTo(((Join2State) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof State2Join){
			for (EpsilonTransition epsilonTransition : getEpsilonTransitions()){
				if(epsilonTransition instanceof State2Join &&
				   ((State2Join) epsilonTransition).getSource().getName().compareTo(((State2Join) transition).getSource().getName())==0 &&
				   ((State2Join)epsilonTransition).getTarget().getName().compareTo(((State2Join) transition).getTarget().getName())==0) 
					return true;

			}
		}else if(transition instanceof LabeledState2State){
			for (LabeledTransition labeledState2State : getLabeledTransitions()){
				if(labeledState2State instanceof LabeledState2State &&
				   ((LabeledState2State) labeledState2State).getSource().getName().compareTo(((LabeledState2State) transition).getSource().getName())==0 &&
				   ((LabeledState2State)labeledState2State).getTarget().getName().compareTo(((LabeledState2State) transition).getTarget().getName())==0 &&
				   ((LabeledState2State)labeledState2State).getLabel().getName().compareTo(((LabeledState2State) transition).getLabel().getName())==0 &&
				   ((LabeledState2State)labeledState2State).getOperationType().getValue()==((LabeledState2State) transition).getOperationType().getValue())
					return true;
				
			}
		}
		return false;
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#removeState2StateWithSourceAndTargetEquals(it.univaq.di.choreos.CLTS.Model)
	 */
	@Override
	public void removeState2StateWithSourceAndTargetEquals (){
		//remove all epsilon state 2 state where source state and target state are equals
		for (State2State state2State : getAllEpsilonState2StateTransitions()) {
			if (state2State.getSource().getName().compareTo(state2State.getTarget().getName())==0)
				getEpsilonTransitions().remove(state2State);
		}
	}
	
	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#removeStateAndTransitionIsolated(it.univaq.di.choreos.CLTS.Model)
	 * assume a model with only the states and transitions
	 */
	@Override
	public void removeStateAndTransitionIsolated (){
		// remove isolate state
		List<State> statesToRemove = new ArrayList<State>();
		for (State state : getStates()) {
			if (getAllTransitionsToState(state).size()==0){
				for (Transition  transition : getAllTransitionsFromState(state)) {
					if (transition instanceof State2State){
						getEpsilonTransitions().remove(transition);
					}else if (transition instanceof State2FinalState){
						getEpsilonTransitions().remove(transition);
					}else if (transition instanceof LabeledState2State){
						getLabeledTransitions().remove(transition);
					}
				}
				statesToRemove.add(state);
			}
		}
		for (State state : statesToRemove) {
			getStates().remove(state);
		}
		
	}

	/* (non-Javadoc)
	 * @see it.univaq.di.choreos.clts.strongsimulation.impl.Helper#copyModel(it.univaq.di.choreos.CLTS.Model)
	 */
	@Override
	public Model copyModel (){
		Copier copier = new Copier();
		EObject result = copier.copy(model);
		copier.copyReferences();
		Model newModel =(Model) result;

		return newModel;
	}

	@Override
	public boolean saveModel(String fullPathDestinationModel) {
		if(model == null)
			return false;

		// create resource from the Model
		URI fileUriTempModelNormalized = URI.createFileURI(fullPathDestinationModel);
		Resource resourceModelNormalized = new XMIResourceFactoryImpl().createResource(fileUriTempModelNormalized);
		// add model in model resourceModel
		resourceModelNormalized.getContents().add(model);
		try {
			//save new ELTS model
			resourceModelNormalized.save(Collections.EMPTY_MAP);
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return true;

	}

}
