package it.univaq.di.choreos.elts.strongsimulation.impl;

import it.univaq.di.choreos.ELTS.ActivityLoop;
import it.univaq.di.choreos.ELTS.Alternative;
import it.univaq.di.choreos.ELTS.FinalState;
import it.univaq.di.choreos.ELTS.Fork;
import it.univaq.di.choreos.ELTS.InitialState;
import it.univaq.di.choreos.ELTS.Join;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.State;

import java.util.HashMap;

public class RepresentationCLTSModel {
	private InitialState initialState;
	private FinalState finalState;
	private HashMap<String, RepresentationNamedElement> representationModel;
	
	public RepresentationCLTSModel(Model model){
		representationModel = new HashMap<String, RepresentationNamedElement>();
		buildRepresentationModel(model);
	}

	public InitialState getInitialState(){
		return initialState;
	}
	
	public FinalState getFinalState(){
		return finalState;
	}
	
	public HashMap<String, RepresentationNamedElement> getRepresentation(){
		return representationModel;
	}

	
	/**
	 * build the adjacency list for representation model
	 */
	private void buildRepresentationModel(Model model){
		//insert states
		for (State state : model.getStates()) {
			RepresentationNamedElement representationState = new RepresentationNamedElement(model, state);
			representationModel.put(state.getClass().getSimpleName()+"_"+state.getName(), representationState);
		}
		//insert activityLoops
		for (ActivityLoop activityLoop : model.getActivityLoops()) {
			RepresentationNamedElement representationActivityLoop = new RepresentationNamedElement(model, activityLoop);
			representationModel.put(activityLoop.getClass().getSimpleName()+"_"+activityLoop.getName(), representationActivityLoop);	
		}
		//insert alternatives
		for (Alternative alternative : model.getAlternatives()) {
			RepresentationNamedElement representationAlternative = new RepresentationNamedElement(model, alternative);
			representationModel.put(alternative.getClass().getSimpleName()+"_"+alternative.getName(), representationAlternative);		
		}
		//insert forks
		for (Fork fork : model.getForks()) {
			RepresentationNamedElement representationFork = new RepresentationNamedElement(model, fork);
			representationModel.put(fork.getClass().getSimpleName()+"_"+fork.getName(), representationFork);
		}
		//insert joins
		for (Join join : model.getJoins()) {
			RepresentationNamedElement representationJoin = new RepresentationNamedElement(model, join);
			representationModel.put(join.getClass().getSimpleName()+"_"+join.getName(), representationJoin);
		}
		//insert initial state
		RepresentationNamedElement representationInitialState = new RepresentationNamedElement(model, model.getInitialState());
		representationModel.put(model.getInitialState().getClass().getSimpleName()+"_"+model.getInitialState().getName(), representationInitialState);
		initialState= model.getInitialState();
		//insert final state
		finalState= model.getFinalState();
	}
}
