package it.univaq.di.choreos.elts.strongsimulation.impl;

import it.univaq.di.choreos.ELTS.ActivityLoop;
import it.univaq.di.choreos.ELTS.ActivityLoop2State;
import it.univaq.di.choreos.ELTS.Alternative;
import it.univaq.di.choreos.ELTS.Alternative2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.Fork;
import it.univaq.di.choreos.ELTS.Fork2State;
import it.univaq.di.choreos.ELTS.InitialState;
import it.univaq.di.choreos.ELTS.InitialState2State;
import it.univaq.di.choreos.ELTS.Join;
import it.univaq.di.choreos.ELTS.Join2State;
import it.univaq.di.choreos.ELTS.LabeledState2State;
import it.univaq.di.choreos.ELTS.LabeledTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.NamedElement;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2ActivityLoop;
import it.univaq.di.choreos.ELTS.State2Alternative;
import it.univaq.di.choreos.ELTS.State2FinalState;
import it.univaq.di.choreos.ELTS.State2Fork;
import it.univaq.di.choreos.ELTS.State2Join;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.ELTS.Transition;

import java.util.ArrayList;
import java.util.List;

public class RepresentationNamedElement {
	private NamedElement element;
	private List<Transition> transitions;
	
	public RepresentationNamedElement (Model model, NamedElement element){
		this.element=element;
		buildTransitions(model);
	}
	
	public RepresentationNamedElement (NamedElement element, List<Transition> transitions){
		this.element=element;
		this.transitions=transitions;
	}
	
	
	/**
	 * 
	 * @return NamedElement
	 */
	public NamedElement getElement() {
		return element;
	}

	/**
	 * 
	 * @return list of outgoing transition from NamedElement
	 */
	public List<Transition> getTransitions() {
		return transitions;
	}
	

 	private void buildTransitions(Model model) {
		if (element instanceof State){
			transitions = getAllTransitionsFromState(model, (State)element);			
		}else if (element instanceof ActivityLoop){
			transitions = getAllTransitionsFromActivityLoop(model, (ActivityLoop)element);
		}else if (element instanceof Alternative){
			transitions = getAllTransitionsFromAlternative(model, (Alternative)element);
		}else if (element instanceof Fork){
			transitions = getAllTransitionsFromFork(model, (Fork)element);
		}else if (element instanceof Join){
			transitions = getAllTransitionsFromJoin(model, (Join)element);
		}else if (element instanceof InitialState){
			transitions = getAllTransitionsFromInitialState(model, (InitialState)element);
		}
	}
	
	/**
	 * 
	 * @param state state
	 * @return all transitions from state
	 */
	private List<Transition> getAllTransitionsFromState (Model model, State state){
		List <Transition> transitionsFromState = new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()){
			if (epsilonTransition instanceof State2State){
				if (((State2State) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2FinalState){
				if (((State2FinalState) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2Fork){
				if (((State2Fork) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}else if (epsilonTransition instanceof State2Join){
				if (((State2Join) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}
			else if (epsilonTransition instanceof State2ActivityLoop){
				if (((State2ActivityLoop) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}
			else if (epsilonTransition instanceof State2Alternative){
				if (((State2Alternative) epsilonTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(epsilonTransition);
			}
		}
		for (LabeledTransition labeledTransition : model.getLabeledTransitions()){
			if (labeledTransition instanceof LabeledState2State){
				if (((LabeledState2State) labeledTransition).getSource().getName().compareTo(state.getName())==0)
					transitionsFromState.add(labeledTransition);
			}
		}
		return transitionsFromState;
	}
	
	/**
	 * 
	 * @param activityLoop activityLoop
	 * @return all transitions from activityLoop
	 */
	private List<Transition> getAllTransitionsFromActivityLoop (Model model, ActivityLoop activityLoop){
		List<Transition> transitionsFromActivityLoop =new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()) {
			if (epsilonTransition instanceof ActivityLoop2State){
				if (((ActivityLoop2State) epsilonTransition).getSource().getName().compareTo(activityLoop.getName())==0)
					transitionsFromActivityLoop.add(epsilonTransition);
			}
		}
		return transitionsFromActivityLoop;
	}
	
	/**
	 * 
	 * @param alternative alternative
	 * @return all transitions from alternative
	 */
	private List<Transition> getAllTransitionsFromAlternative (Model model, Alternative alternative){
		List<Transition> transitionsFromAlternative =new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()) {
			if (epsilonTransition instanceof Alternative2State){
				if (((Alternative2State) epsilonTransition).getSource().getName().compareTo(alternative.getName())==0)
					transitionsFromAlternative.add(epsilonTransition);
			}
		}
		return transitionsFromAlternative;
	}
	
	/**
	 * 
	 * @param fork fork
	 * @return all transitions from fork
	 */
	private List<Transition> getAllTransitionsFromFork (Model model, Fork fork){
		List<Transition> transitionsFromFork=new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()) {
			if  (epsilonTransition instanceof Fork2State){
				if (((Fork2State) epsilonTransition).getSource().getName().compareTo(fork.getName())==0)
					transitionsFromFork.add(epsilonTransition);
			}
			
		}
		return transitionsFromFork;
	}

	/**
	 * 
	 * @param join join
	 * @return all transitions from join
	 */
	private List<Transition> getAllTransitionsFromJoin (Model model, Join join){
		List<Transition> transitionsFromJoin=new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()) {
			if  (epsilonTransition instanceof Join2State){
				if (((Join2State) epsilonTransition).getSource().getName().compareTo(join.getName())==0)
					transitionsFromJoin.add(epsilonTransition);
			}
		}
		return transitionsFromJoin;
	}

	/**
	 * 
	 * @param initalState initalState
	 * @return all transitions from initalState
	 */
	private List<Transition> getAllTransitionsFromInitialState (Model model, InitialState initalState){
		List<Transition> transitionsInitialState=new ArrayList<Transition>();
		for (EpsilonTransition epsilonTransition : model.getEpsilonTransitions()) {
			if (epsilonTransition instanceof InitialState2State){
				if (((InitialState2State) epsilonTransition).getSource().getName().compareTo(initalState.getName())==0)
					transitionsInitialState.add(epsilonTransition);
			}
		}
		return transitionsInitialState;
	}

	
}
