package it.univaq.di.choreos.bpmn2clts;


import it.univaq.di.choreos.CLTS.Alternative;
import it.univaq.di.choreos.CLTS.FinalState;
import it.univaq.di.choreos.CLTS.Fork;
import it.univaq.di.choreos.CLTS.InitialState;
import it.univaq.di.choreos.CLTS.Join;
import it.univaq.di.choreos.CLTS.Model;
import it.univaq.di.choreos.CLTS.Operation;
import it.univaq.di.choreos.CLTS.Role;
import it.univaq.di.choreos.CLTS.State;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.bpmn2.ChoreographyTask;
import org.eclipse.bpmn2.EndEvent;
import org.eclipse.bpmn2.ExclusiveGateway;
import org.eclipse.bpmn2.FlowElement;
import org.eclipse.bpmn2.FlowNode;
import org.eclipse.bpmn2.GatewayDirection;
import org.eclipse.bpmn2.ParallelGateway;
import org.eclipse.bpmn2.SequenceFlow;
import org.eclipse.bpmn2.StartEvent;

/*
 * ChoreographyTask *
 * SequenceFlow *
 * ExclusiveGateway *
 * 
 */
public class Bpmn2CltsTransformation {
	private BpmnManager bpmnManager;
	private CltsManager cltsManager;
	private Map<FlowElement, RepresentationBpmnObject> objects;
	private List<ChoreographyTask> choreographyTasks;

	public Bpmn2CltsTransformation() {
		cltsManager = new CltsManager();
		choreographyTasks = new ArrayList<ChoreographyTask>();
		objects = new HashMap<FlowElement, RepresentationBpmnObject>();
	}

	public Model transform(String bpmnPath) throws Bpmn2CltsException {
		bpmnManager = new BpmnManager(bpmnPath);
		createRepresentationObject();
		createCltsElement();
		return cltsManager.getModel();
	}

	private void createRepresentationObject() {
		for (FlowElement flowElement : bpmnManager.getElements().values()) {
			if (flowElement instanceof StartEvent) {
				objects.put(flowElement, getRepresentationStartEvent((StartEvent) flowElement));

			} else if (flowElement instanceof EndEvent) {
				objects.put(flowElement, getRepresentationEndEvent((EndEvent) flowElement));

			} else if (flowElement instanceof ChoreographyTask) {
				objects.put(flowElement, getRepresentationChoreographyTask((ChoreographyTask) flowElement));

			} else if (flowElement instanceof ExclusiveGateway) {
				objects.put(flowElement, getRepresentationExclusiveGateway((ExclusiveGateway) flowElement));

			} else if (flowElement instanceof ParallelGateway) {
				objects.put(flowElement, getRepresentationParallelGateway((ParallelGateway) flowElement));

			}

		}
	}

	private RepresentationBpmnObject getRepresentationStartEvent(StartEvent eleEvent) {
		return new RepresentationBpmnObject(eleEvent, cltsManager.createInitialState(), null);
	}

	private RepresentationBpmnObject getRepresentationEndEvent(EndEvent eleEvent) {
		return new RepresentationBpmnObject(eleEvent, cltsManager.createFinalState(), null);
	}

	private RepresentationBpmnObject getRepresentationChoreographyTask(ChoreographyTask eleEvent) {
		return new RepresentationBpmnObject(eleEvent, cltsManager.createState(), cltsManager.createState());
	}

	private RepresentationBpmnObject getRepresentationExclusiveGateway(ExclusiveGateway eleEvent) {
		if (eleEvent.getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			return new RepresentationBpmnObject(eleEvent, cltsManager.createAlternative(), null);
		} else {
			return new RepresentationBpmnObject(eleEvent, cltsManager.createState(), null);
		}

	}

	private RepresentationBpmnObject getRepresentationParallelGateway(ParallelGateway eleEvent) {
		if (eleEvent.getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			return new RepresentationBpmnObject(eleEvent, cltsManager.createFork(), null);
		} else {
			return new RepresentationBpmnObject(eleEvent, cltsManager.createJoin(), null);
		}
	}

	private void createCltsElement() {
		for (FlowElement flowElement : bpmnManager.getElements().values()) {
			if (flowElement instanceof SequenceFlow) {
				selectSource((SequenceFlow) flowElement);
			}
		}
	}

	private void selectSource(SequenceFlow sequenceFlow) {
		FlowNode source = sequenceFlow.getSourceRef();
		if (source instanceof StartEvent) {
			addStartEventToUndefinedElement((StartEvent) source, sequenceFlow);
		} else if (source instanceof ChoreographyTask) {
			addChoreographyTaskToUndefinedElement((ChoreographyTask) source, sequenceFlow);
		} else if (source instanceof ExclusiveGateway && ((ExclusiveGateway) source).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			addExclusiveGatewayDivergingToUndefinedElement((ExclusiveGateway) source, sequenceFlow);
		} else if (source instanceof ExclusiveGateway && ((ExclusiveGateway) source).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			addExclusiveGatewayConvergingToUndefinedElement((ExclusiveGateway) source, sequenceFlow);
		} else if (source instanceof ParallelGateway && ((ParallelGateway) source).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			addParallelGatewayDivergingToUndefinedElement((ParallelGateway) source, sequenceFlow);
		} else if (source instanceof ParallelGateway && ((ParallelGateway) source).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			addParallelGatewayConvergingToUndefinedElement((ParallelGateway) source, sequenceFlow);
		}
	}

	private void addStartEventToUndefinedElement(StartEvent source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			State state = cltsManager.createState();
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2FinalState(state, (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {
			addChoreographyTask((ChoreographyTask) target);
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) source).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Alternative(state, (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) source).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) source).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Fork(state, (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) source).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addInitialState2State((InitialState) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Join(state, (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addChoreographyTaskToUndefinedElement(ChoreographyTask source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			cltsManager.addState2FinalState((State) objects.get(source).getSecondCltsElement(), (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {

			if (choreographyTasks.contains(source) && choreographyTasks.contains(target)) {
				cltsManager.addState2State((State) objects.get(source).getSecondCltsElement(), (State) objects.get(target).getFirstCltsElement());
			} else if (choreographyTasks.contains(source) && !choreographyTasks.contains(target)) {
				Operation operation = cltsManager.getOperation(target.getName());
				Role sourceParticipant = cltsManager.getParticipant(bpmnManager.getSourceParticipantName((ChoreographyTask) target));
				Role targetParticipant = cltsManager.getParticipant(bpmnManager.getTargetParticipantName((ChoreographyTask) target));
				cltsManager.addLabeledState2StateTransition(operation, sourceParticipant, targetParticipant, (State) objects.get(source).getSecondCltsElement(), (State) objects.get(target).getSecondCltsElement());
				choreographyTasks.add((ChoreographyTask) target);
			} else if (!choreographyTasks.contains(source) && choreographyTasks.contains(target)) {
				Operation operation = cltsManager.getOperation(source.getName());
				Role sourceParticipant = cltsManager.getParticipant(bpmnManager.getSourceParticipantName(source));
				Role targetParticipant = cltsManager.getParticipant(bpmnManager.getTargetParticipantName(source));
				cltsManager.addLabeledState2StateTransition(operation, sourceParticipant, targetParticipant, (State) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
				choreographyTasks.add(source);
			} else {
				addChoreographyTask(source);
				Operation operation = cltsManager.getOperation(target.getName());
				Role sourceParticipant = cltsManager.getParticipant(bpmnManager.getSourceParticipantName((ChoreographyTask) target));
				Role targetParticipant = cltsManager.getParticipant(bpmnManager.getTargetParticipantName((ChoreographyTask) target));
				cltsManager.addLabeledState2StateTransition(operation, sourceParticipant, targetParticipant, (State) objects.get(source).getSecondCltsElement(), (State) objects.get(target).getSecondCltsElement());
				choreographyTasks.add((ChoreographyTask) target);
			}

		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			cltsManager.addState2Alternative((State) objects.get(source).getSecondCltsElement(), (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addState2State((State) objects.get(source).getSecondCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			cltsManager.addState2Fork((State) objects.get(source).getSecondCltsElement(), (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addState2Join((State) objects.get(source).getSecondCltsElement(), (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addExclusiveGatewayDivergingToUndefinedElement(ExclusiveGateway source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			State state = cltsManager.createState();
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2FinalState(state, (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {
			addChoreographyTask((ChoreographyTask) target);
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Alternative(state, (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Fork(state, (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addAlternative2State((Alternative) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Join(state, (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addExclusiveGatewayConvergingToUndefinedElement(ExclusiveGateway source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			cltsManager.addState2FinalState((State) objects.get(source).getFirstCltsElement(), (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {
			if (!choreographyTasks.contains(target)) {
				Operation operation = cltsManager.getOperation(target.getName());
				Role sourceParticipant = cltsManager.getParticipant(bpmnManager.getSourceParticipantName((ChoreographyTask) target));
				Role targetParticipant = cltsManager.getParticipant(bpmnManager.getTargetParticipantName((ChoreographyTask) target));
				cltsManager.addLabeledState2StateTransition(operation, sourceParticipant, targetParticipant, (State) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getSecondCltsElement());
				choreographyTasks.add((ChoreographyTask) target);
			} else {
				cltsManager.addState2State((State) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
			}
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			cltsManager.addState2Alternative((State) objects.get(source).getFirstCltsElement(), (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addState2State((State) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			cltsManager.addState2Fork((State) objects.get(source).getFirstCltsElement(), (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addState2Join((State) objects.get(source).getFirstCltsElement(), (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addParallelGatewayDivergingToUndefinedElement(ParallelGateway source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			State state = cltsManager.createState();
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2FinalState(state, (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {
			addChoreographyTask((ChoreographyTask) target);
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Alternative(state, (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Fork(state, (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addFork2State((Fork) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Join(state, (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addParallelGatewayConvergingToUndefinedElement(ParallelGateway source, SequenceFlow sequenceFlow) {
		FlowNode target = sequenceFlow.getTargetRef();
		if (target instanceof EndEvent) {
			State state = cltsManager.createState();
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2FinalState(state, (FinalState) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ChoreographyTask) {
			addChoreographyTask((ChoreographyTask) target);
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Alternative(state, (Alternative) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ExclusiveGateway && ((ExclusiveGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), (State) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.DIVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Fork(state, (Fork) objects.get(target).getFirstCltsElement());
		} else if (target instanceof ParallelGateway && ((ParallelGateway) target).getGatewayDirection().getValue() == GatewayDirection.CONVERGING_VALUE) {
			State state = cltsManager.createState();
			cltsManager.addJoin2State((Join) objects.get(source).getFirstCltsElement(), state);
			cltsManager.addState2Join(state, (Join) objects.get(target).getFirstCltsElement());
		}
	}

	private void addChoreographyTask(ChoreographyTask choreographyTask) {
		if (!choreographyTasks.contains(choreographyTask)) {
			Operation operation = cltsManager.getOperation(choreographyTask.getName());
			Role sourceParticipant = cltsManager.getParticipant(bpmnManager.getSourceParticipantName(choreographyTask));
			Role targetParticipant = cltsManager.getParticipant(bpmnManager.getTargetParticipantName(choreographyTask));
			cltsManager.addLabeledState2StateTransition(operation, sourceParticipant, targetParticipant, (State) objects.get(choreographyTask).getFirstCltsElement(), (State) objects.get(choreographyTask).getSecondCltsElement());
			choreographyTasks.add(choreographyTask);
		}
	}

}
