package it.univaq.di.choreos.bpmn2clts;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


import org.eclipse.bpmn2.Choreography;
import org.eclipse.bpmn2.ChoreographyTask;
import org.eclipse.bpmn2.Definitions;
import org.eclipse.bpmn2.FlowElement;
import org.eclipse.bpmn2.Participant;
import org.eclipse.bpmn2.RootElement;
import org.eclipse.bpmn2.util.Bpmn2ResourceFactoryImpl;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

public class BpmnManager {
	private Map<String, FlowElement> elements;
	
	public BpmnManager(String bpmnPath) throws Bpmn2CltsException {
		elements = new HashMap<String, FlowElement>();

		try {
			URI uri = URI.createURI(bpmnPath);
			// Register the BPMN2ResourceFactory in Factory registry
			Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
			reg.getExtensionToFactoryMap().put("bpmn2", new Bpmn2ResourceFactoryImpl());
			// load the resource and resolve
			ResourceSet rs = new ResourceSetImpl();
			Resource res = rs.createResource(uri);
			// load it
			res.load(null);
			Definitions definition = (Definitions) res.getContents().get(0).eContents().get(0);
			if (definition == null) {
				throw new Bpmn2CltsException("The Choreography element is not present");
			}
			Choreography choreography = getChoreography(definition);
			initElements(choreography.getFlowElements());
		} catch (IOException e) {
			throw new Bpmn2CltsException("Error to load Bpmn2");
		}

	}
	
	private Choreography getChoreography (Definitions d){
		for (RootElement element : d.getRootElements()) {
			if (element instanceof Choreography)
				return (Choreography)element;
		}
		return null;
	}
	
	private void initElements (List<FlowElement> flowElements){
		for (FlowElement flowElement : flowElements) {
			elements.put(flowElement.getId(), flowElement);
		}
	}
	
	
	public String getSourceParticipantName (ChoreographyTask choreographyTask){
		return choreographyTask.getInitiatingParticipantRef().getName();
	}
	
	public String getTargetParticipantName (ChoreographyTask choreographyTask){
		for (Participant participant : choreographyTask.getParticipantRefs()) {
			if (!participant.equals(choreographyTask.getInitiatingParticipantRef())){
				return participant.getName();
			}
		}
		return null;
	}
	
	
	public Map<String, FlowElement> getElements(){
		return elements;
	}
	
	
	
	
}
