package it.univaq.di.choreos.bpmn2clts;

import it.univaq.di.choreos.CLTS.ActivityLoop;
import it.univaq.di.choreos.CLTS.ActivityLoop2State;
import it.univaq.di.choreos.CLTS.Alternative;
import it.univaq.di.choreos.CLTS.Alternative2State;
import it.univaq.di.choreos.CLTS.CLTSFactory;
import it.univaq.di.choreos.CLTS.FinalState;
import it.univaq.di.choreos.CLTS.Fork;
import it.univaq.di.choreos.CLTS.Fork2State;
import it.univaq.di.choreos.CLTS.InitialState;
import it.univaq.di.choreos.CLTS.InitialState2State;
import it.univaq.di.choreos.CLTS.Join;
import it.univaq.di.choreos.CLTS.Join2State;
import it.univaq.di.choreos.CLTS.LabeledState2State;
import it.univaq.di.choreos.CLTS.Model;
import it.univaq.di.choreos.CLTS.Operation;
import it.univaq.di.choreos.CLTS.Role;
import it.univaq.di.choreos.CLTS.State;
import it.univaq.di.choreos.CLTS.State2ActivityLoop;
import it.univaq.di.choreos.CLTS.State2Alternative;
import it.univaq.di.choreos.CLTS.State2FinalState;
import it.univaq.di.choreos.CLTS.State2Fork;
import it.univaq.di.choreos.CLTS.State2Join;
import it.univaq.di.choreos.CLTS.State2State;


public class CltsManager {

	private Model model;
	private int countStateName;
	
	public CltsManager (){
		model = CLTSFactory.eINSTANCE.createModel();
		model.setOperationsDefinition(CLTSFactory.eINSTANCE.createOperationsDefinition());
		model.setRolesDefinition(CLTSFactory.eINSTANCE.createRolesDefinition());
		countStateName = 0;
	}
	
	public Model getModel (){
		return model;
	}
	
	private String getUniqueElementName(){
		countStateName++;
		return "S"+countStateName;
		
	}
	
	public Role getParticipant (String name){
		for (Role role : model.getRolesDefinition().getRoles()) {
			if (role.getName().equals(name)){
				return role;
			}
		}
		Role role =	CLTSFactory.eINSTANCE.createRole();
		role.setName(name);
		return role;
	}
	
	public Operation getOperation(String name){
		for (Operation operation : model.getOperationsDefinition().getOperations()) {
			if (operation.getName().equals(name)){
				return operation;
			}
		}
		Operation operation =CLTSFactory.eINSTANCE.createOperation();
		operation.setName(name);
		return operation;
	}
	
	/*
	 * create NamedElement
	 */
	
	public InitialState createInitialState() {
		InitialState initialState = CLTSFactory.eINSTANCE.createInitialState();
		initialState.setName(getUniqueElementName());
		return initialState;
	}
	
	public FinalState createFinalState() {
		FinalState finalState = CLTSFactory.eINSTANCE.createFinalState();
		finalState.setName(getUniqueElementName());
		return finalState;
	}
	
	public State createState() {
		State newState= CLTSFactory.eINSTANCE.createState();
		newState.setName(getUniqueElementName());
		return newState;
	}

	
	public Fork createFork() {
		Fork newFork= CLTSFactory.eINSTANCE.createFork();
		newFork.setName(getUniqueElementName());
		return newFork;
	}

	
	public Join createJoin() {
		Join newJoin= CLTSFactory.eINSTANCE.createJoin();
		newJoin.setName(getUniqueElementName());
		return newJoin;
	}

	
	public Alternative createAlternative() {
		Alternative newAlternative= CLTSFactory.eINSTANCE.createAlternative();
		newAlternative.setName(getUniqueElementName());
		return newAlternative;
	}

	
	public ActivityLoop createActivityLoop() {
		ActivityLoop newActivityLoop= CLTSFactory.eINSTANCE.createActivityLoop();
		newActivityLoop.setName(getUniqueElementName());
		return newActivityLoop;
	}
	
	
	/*
	 * create Labeled Transition
	 */

	
	
	public LabeledState2State createLabeledState2State(State source, State target, Role partecipant, Role initiatingParticipant, Operation operation) {
		LabeledState2State newLabeledState2State= CLTSFactory.eINSTANCE.createLabeledState2State();
		newLabeledState2State.setSource(source);
		newLabeledState2State.setTarget(target);
		newLabeledState2State.setInitiatingParticipant(initiatingParticipant);
		newLabeledState2State.setParticipant(partecipant);
		newLabeledState2State.setLabel(operation);
		return newLabeledState2State;
	}

	
	
	
	/*
	 * create Epsilon Transition
	 */
	
	
	public Join2State createJoin2State(Join source, State target) {
		Join2State newJoin2State = CLTSFactory.eINSTANCE.createJoin2State();
		newJoin2State.setSource(source);
		newJoin2State.setTarget(target);
		return newJoin2State;
	}

	
	public State2Fork createState2Fork(State source, Fork target) {
		State2Fork newState2Fork = CLTSFactory.eINSTANCE.createState2Fork();
		newState2Fork.setSource(source);
		newState2Fork.setTarget(target);
		return newState2Fork;
	}

	
	public Fork2State createFork2State(Fork source, State target) {
		Fork2State newFork2State = CLTSFactory.eINSTANCE.createFork2State();
		newFork2State.setSource(source);
		newFork2State.setTarget(target);
		return newFork2State;
	}

	

	
	public State2Join createState2Join(State source, Join target) {
		State2Join newState2Join = CLTSFactory.eINSTANCE.createState2Join();
		newState2Join.setSource(source);
		newState2Join.setTarget(target);
		return newState2Join;
	}

	

	
	public InitialState2State createInitialState2State(InitialState source, State target) {
		InitialState2State newInitialState2State = CLTSFactory.eINSTANCE.createInitialState2State();
		newInitialState2State.setSource(source);
		newInitialState2State.setTarget(target);
		return newInitialState2State;
	}

	
	public State2FinalState createState2FinalState(State source, FinalState target) {
		State2FinalState newState2FinalState = CLTSFactory.eINSTANCE.createState2FinalState();
		newState2FinalState.setSource(source);
		newState2FinalState.setTarget(target);
		return newState2FinalState;
	}

	
	public State2Alternative createState2Alternative(State source, Alternative target) {
		State2Alternative newState2Alternative = CLTSFactory.eINSTANCE.createState2Alternative();
		newState2Alternative.setSource(source);
		newState2Alternative.setTarget(target);
		return newState2Alternative;
	}

	
	public Alternative2State createAlternative2State(Alternative source, State target) {
		Alternative2State newAlternative2State = CLTSFactory.eINSTANCE.createAlternative2State();
		newAlternative2State.setSource(source);
		newAlternative2State.setTarget(target);
		return newAlternative2State;
	}

	
	public State2ActivityLoop createState2ActivityLoop(State source, ActivityLoop target) {
		State2ActivityLoop newState2ActivityLoop = CLTSFactory.eINSTANCE.createState2ActivityLoop();
		newState2ActivityLoop.setSource(source);
		newState2ActivityLoop.setTarget(target);
		return newState2ActivityLoop;
	}

	
	public ActivityLoop2State createActivityLoop2State(ActivityLoop source, State target) {
		ActivityLoop2State newActivityLoop2State = CLTSFactory.eINSTANCE.createActivityLoop2State();
		newActivityLoop2State.setSource(source);
		newActivityLoop2State.setTarget(target);
		return newActivityLoop2State;
	}

	

	
	public State2State createState2State(State source, State target) {
		State2State newState2State = CLTSFactory.eINSTANCE.createState2State();
		newState2State.setSource(source);
		newState2State.setTarget(target);
		return newState2State;
	}
	
	public LabeledState2State addLabeledState2StateTransition(Operation operation, Role sourceParticipant, Role targetParticipant, State sourceState, State targetState) {
		if (!model.getOperationsDefinition().getOperations().contains(operation)) {
			model.getOperationsDefinition().getOperations().add(operation);
		}
		if (!model.getRolesDefinition().getRoles().contains(sourceParticipant)) {
			model.getRolesDefinition().getRoles().add(sourceParticipant);
		}
		if (!model.getRolesDefinition().getRoles().contains(targetParticipant)) {
			model.getRolesDefinition().getRoles().add(targetParticipant);
		}
		if (!model.getStates().contains(sourceState)) {
			model.getStates().add(sourceState);
		}
		if (!model.getStates().contains(targetState)) {
			model.getStates().add(targetState);
		}
		LabeledState2State transition = CLTSFactory.eINSTANCE.createLabeledState2State();
		transition.setInitiatingParticipant(sourceParticipant);
		transition.setParticipant(targetParticipant);
		transition.setLabel(operation);
		transition.setSource(sourceState);
		transition.setTarget(targetState);
		model.getLabeledTransitions().add(transition);

		return transition;
	}
	
	public Join2State addJoin2State(Join source, State target) {
		if (!model.getJoins().contains(source)) {
			model.getJoins().add(source);
		}
		if (!model.getStates().contains(target)) {
			model.getStates().add(target);
		}
		Join2State newJoin2State = CLTSFactory.eINSTANCE.createJoin2State();
		newJoin2State.setSource(source);
		newJoin2State.setTarget(target);
		model.getEpsilonTransitions().add(newJoin2State);
		return newJoin2State;
	}

	public State2Fork addState2Fork(State source, Fork target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (!model.getForks().contains(target)) {
			model.getForks().add(target);
		}
		State2Fork newState2Fork = CLTSFactory.eINSTANCE.createState2Fork();
		newState2Fork.setSource(source);
		newState2Fork.setTarget(target);
		model.getEpsilonTransitions().add(newState2Fork);
		return newState2Fork;
	}

	public Fork2State addFork2State(Fork source, State target) {
		if (!model.getForks().contains(source)) {
			model.getForks().add(source);
		}
		if (!model.getStates().contains(target)) {
			model.getStates().add(target);
		}
		Fork2State newFork2State = CLTSFactory.eINSTANCE.createFork2State();
		newFork2State.setSource(source);
		newFork2State.setTarget(target);
		model.getEpsilonTransitions().add(newFork2State);
		return newFork2State;
	}

	public State2Join addState2Join(State source, Join target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (!model.getJoins().contains(target)) {
			model.getJoins().add(target);
		}
		State2Join newState2Join = CLTSFactory.eINSTANCE.createState2Join();
		newState2Join.setSource(source);
		newState2Join.setTarget(target);
		model.getEpsilonTransitions().add(newState2Join);
		return newState2Join;
	}
	
	public InitialState2State addInitialState2State(InitialState source, State target) {
		if (model.getInitialState() == null) {
			model.setInitialState(source);
		}
		if (!model.getStates().contains(target)) {
			model.getStates().add(target);
		}
		InitialState2State newInitialState2State = CLTSFactory.eINSTANCE.createInitialState2State();
		newInitialState2State.setSource(source);
		newInitialState2State.setTarget(target);
		model.getEpsilonTransitions().add(newInitialState2State);
		return newInitialState2State;
	}

	public State2FinalState addState2FinalState(State source, FinalState target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (model.getFinalState() == null) {
			model.setFinalState(target);
		}
		State2FinalState transition = CLTSFactory.eINSTANCE.createState2FinalState();
			transition.setSource(source);
			transition.setTarget(target);
			model.getEpsilonTransitions().add(transition);
		
		
		return transition;
	}
	
	public State2Alternative addState2Alternative(State source, Alternative target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (!model.getAlternatives().contains(target)) {
			model.getAlternatives().add(target);
		}
		State2Alternative newState2Alternative = CLTSFactory.eINSTANCE.createState2Alternative();
		newState2Alternative.setSource(source);
		newState2Alternative.setTarget(target);
		model.getEpsilonTransitions().add(newState2Alternative);
		return newState2Alternative;
	}

	public Alternative2State addAlternative2State(Alternative source, State target) {
		if (!model.getAlternatives().contains(source)) {
			model.getAlternatives().add(source);
		}
		if (!model.getStates().contains(target)) {
			model.getStates().add(target);
		}
		Alternative2State newAlternative2State = CLTSFactory.eINSTANCE.createAlternative2State();
		newAlternative2State.setSource(source);
		newAlternative2State.setTarget(target);
		model.getEpsilonTransitions().add(newAlternative2State);
		return newAlternative2State;
	}

	public State2ActivityLoop addState2ActivityLoop(State source, ActivityLoop target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (!model.getActivityLoops().contains(target)) {
			model.getActivityLoops().add(target);
		}
		State2ActivityLoop newState2ActivityLoop = CLTSFactory.eINSTANCE.createState2ActivityLoop();
		newState2ActivityLoop.setSource(source);
		newState2ActivityLoop.setTarget(target);
		model.getEpsilonTransitions().add(newState2ActivityLoop);
		return newState2ActivityLoop;
	}

	public ActivityLoop2State addActivityLoop2State(ActivityLoop source, State target) {
		if (!model.getActivityLoops().contains(source)) {
			model.getActivityLoops().add(source);
		}
		if (!model.getStates().contains(target)) {
			model.getStates().add(target);
		}
		ActivityLoop2State newActivityLoop2State = CLTSFactory.eINSTANCE.createActivityLoop2State();
		newActivityLoop2State.setSource(source);
		newActivityLoop2State.setTarget(target);
		model.getEpsilonTransitions().add(newActivityLoop2State);
		return newActivityLoop2State;
	}

	public State2State addState2State(State source, State target) {
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		if (!model.getStates().contains(source)) {
			model.getStates().add(source);
		}
		State2State newState2State = CLTSFactory.eINSTANCE.createState2State();
		newState2State.setSource(source);
		newState2State.setTarget(target);
		model.getEpsilonTransitions().add(newState2State);
		return newState2State;
	}	


	
}
