package it.univaq.di.choreos.stockconfirm.fineco.cd;


import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationChannelManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateData;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.DiscardException;
import it.univaq.di.choreos.coordinationprotocol.algorithm.ForwardAndDiscardMessagesCallback;
import it.univaq.di.choreos.coordinationprotocol.algorithm.impl.CoordinationChannelManagerImpl;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.ContextData;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelParser;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@WebService( serviceName="StockConfirm", targetNamespace="http://choreos.di.univaq.it", portName="StockConfirmPort" )
public class StockConfirm implements ForwardAndDiscardMessagesCallback {
	
	static final Logger logger = LoggerFactory.getLogger(StockConfirm.class);

	private static AllowedOperation requestOperation;
	private static Participant coordinationDelegateFrom;

	private static final String COORDMODEL_FILENAME = "fineco.coordmodel";
	private static final String COORDINATION_DELEGATE_URL = "coordinationdelegatesurl.properties";
	private static final String REQUEST_OPERATION = "orderconfirm";
	private static final String INITIAL_STATE = "0";
	private static final int INITIAL_TIMESTAMP = 0;
	private static final String COORDINATION_DELEGATE_FROM = "fineco-cd";
	private static final String COORDINATION_DELEGATE_TYPE = "stockconfirm-cd";
	private static final String CHOREOGRAPHY_ID = "cd01";
	
	private static final CoordinationDelegateID ID = new CoordinationDelegateID(COORDINATION_DELEGATE_TYPE,CHOREOGRAPHY_ID);
	
	static {
	
		//Creating the CoordinationChannelManager
		String participantsurlFileName = StockConfirm.class.getResource(COORDINATION_DELEGATE_URL).getFile();		
		Map<String, String> coordinationDelegatesUrl = new HashMap<String, String>();
		Properties properties = new Properties();
		try {
			properties.load(new FileReader(participantsurlFileName));
			Set<Map.Entry<Object, Object>> elements = properties.entrySet();
			for (Map.Entry<Object, Object> entry : elements) {
				coordinationDelegatesUrl.put((String)entry.getKey(), (String)entry.getValue());
			}
		} catch (FileNotFoundException e) {
			logger.error("coordinationdelegatesfileproperties", e);
		} catch (IOException e) {
			logger.error("coordinationdelegatesfileproperties", e);
		}
		CoordinationChannelManager coordinationChannelManager = new CoordinationChannelManagerImpl(coordinationDelegatesUrl);
		CoordinationDelegateFactory.setCoordinationChannelManager(coordinationChannelManager);
		
		//Loading Model
		String coordFileName = StockConfirm.class.getResource(COORDMODEL_FILENAME ).getFile();
		Coord coord = CoordmetamodelParser.parse(coordFileName);
		logger.debug(coord.toString());
		
		State initialState = CoordmetamodelFactory.eINSTANCE.createState();
		initialState.setName(INITIAL_STATE);
		int initialTimeStamp = INITIAL_TIMESTAMP;
		CoordinationDelegateData coordinationDelegateData =  new CoordinationDelegateData();
		coordinationDelegateData.setCoordModel(coord);
		coordinationDelegateData.setInitialState(initialState);
		coordinationDelegateData.setInitialTimeStamp(initialTimeStamp);
		CoordinationDelegateFactory.addCoordinationDelegateData(COORDINATION_DELEGATE_TYPE, coordinationDelegateData);
	}

	static {
		requestOperation = CoordmetamodelFactory.eINSTANCE.createAllowedOperation();
		requestOperation.setName(REQUEST_OPERATION);
		coordinationDelegateFrom = CoordmetamodelFactory.eINSTANCE.createParticipant();
		coordinationDelegateFrom.setName(COORDINATION_DELEGATE_FROM );
	}
	
	
	@WebMethod( operationName="orderconfirm" )
	//@Oneway
	public void orderconfirm( @WebParam( name="orderNumber" ) String orderNumber, @WebParam( name="quantity" ) int quantity ) {
		logger.debug("begin orderconfirm");
		OrderConfirmData orderConfirmData = new OrderConfirmData(orderNumber, quantity);
		ContextData contextData = new ContextData(orderConfirmData);
		CoordinationDelegateAlgorithm algorithm = CoordinationDelegateFactory.getCoordinationDelegateAlgorithm(ID);
		algorithm.handleRules(requestOperation, coordinationDelegateFrom, this, contextData); 
		logger.debug("end orderconfirm");
	}

	
	public void forwardMessage(ContextData contextData) {
		OrderConfirmData orderConfirmData = (OrderConfirmData)contextData.getData();
		logger.debug("forwardMessage: " + orderConfirmData);
		
	}

	public void discardMessage(ContextData contextData) throws DiscardException {
		OrderConfirmData orderConfirmData = (OrderConfirmData)contextData.getData();
		logger.debug("discardMessage: " + orderConfirmData);
		
	}

	class OrderConfirmData {
		private  String orderNumber;
		private int quantity;
		
		public OrderConfirmData(String orderNumber, int quantity) {
			super();
			this.orderNumber = orderNumber;
			this.quantity = quantity;
		}
		
		public String getOrderNumber() {
			return orderNumber;
		}
		
		public int getQuantity() {
			return quantity;
		}

		@Override
		public String toString() {
			return "OrderConfirmData [orderNumber=" + orderNumber + ", quantity=" + quantity + "]";
		}
	}

}