package it.univaq.di.choreos.synthesis.m2mtransformator.impl;

import it.univaq.di.choreos.CLTS.CLTSMetamodelParser;
import it.univaq.di.choreos.CLTS.Model;
import it.univaq.di.choreos.CLTS.Role;
import it.univaq.di.choreos.CLTS.RolesDefinition;
import it.univaq.di.choreos.bpmn2clts.Bpmn2CltsException;
import it.univaq.di.choreos.bpmn2clts.Bpmn2CltsTransformation;
import it.univaq.di.choreos.coordderivator.CoordDerivator;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelParser;
import it.univaq.di.choreos.synthesis.datamodel.AbstractParticipant;
import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ChoreographyBPMNSpecification;
import it.univaq.di.choreos.synthesis.datamodel.CoordModel;
import it.univaq.di.choreos.synthesis.datamodel.Participant;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformator;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformatorException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class M2MTransformatorImpl implements M2MTransformator {
	private static final String CLTS_WP6 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp6.clts";
	private static final String CLTS_WP7 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp7-concrete.clts";
	private static final String CLTS_WP8 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp8-concrete.clts";

	private static Logger logger = LoggerFactory.getLogger(M2MTransformatorImpl.class);

	public CLTSModel bpmn2clts(ChoreographyBPMNSpecification bpmnchoreographyspecification) throws M2MTransformatorException {
		logger.info("start bpmn2clts=" + bpmnchoreographyspecification.getChoreographyName());
		CLTSModel cltsContentModel = new CLTSModel();
		cltsContentModel.setChoreographyName(bpmnchoreographyspecification.getChoreographyName());
		if (bpmnchoreographyspecification.getChoreographyName().toLowerCase().contains("wp6")) {
			try {
				cltsContentModel.setCltsModel(FileUtils.readFileToString(new File(CLTS_WP6)));
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			try {
				String tempFolder = createTempFolder();
				File tempBpmn = new File(tempFolder + File.separator + "temp.bpmn2");
				FileUtils.writeStringToFile(tempBpmn, bpmnchoreographyspecification.getBpmnSpecification());
				Bpmn2CltsTransformation bpmn2CltsTransformation = new Bpmn2CltsTransformation();
				Model cltsModel = bpmn2CltsTransformation.transform(tempBpmn.getAbsolutePath());
				FileUtils.deleteQuietly(new File(tempFolder));
				cltsContentModel.setCltsModel(CLTSMetamodelParser.getCltsContent(cltsModel));
			} catch (Bpmn2CltsException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		logger.info("end bpmn2clts");
		return cltsContentModel;
	}

	@Override
	public List<Participant> clts2coords(CLTSModel cltsModel) throws M2MTransformatorException {
		logger.info("start clts2coords");
		Model clts = CLTSMetamodelParser.getCltsModel(cltsModel.getCltsModel());
		CoordDerivator cdDerivator = new CoordDerivator(clts);
		cdDerivator.init();
		List<Participant> result = createParticipant(cdDerivator.getCoords());
		mergeRolesToParticipants(result, clts.getRolesDefinition());
		logger.info("end clts2coords");
		return result;

	}

	@Override
	public CLTSModel abstract2ConcreteCLTSModel(CLTSModel cltsModel, List<AbstractParticipant> participants) throws M2MTransformatorException {
		CLTSModel concreteCltsModel = new CLTSModel();
		concreteCltsModel.setChoreographyName(cltsModel.getChoreographyName());
		try {
			if (cltsModel.getChoreographyName().toLowerCase().contains("wp6")) {
				concreteCltsModel.setCltsModel(FileUtils.readFileToString(new File(CLTS_WP6)));
			} else if (cltsModel.getChoreographyName().toLowerCase().contains("wp7")) {
				concreteCltsModel.setCltsModel(FileUtils.readFileToString(new File(CLTS_WP7)));
			} else if (cltsModel.getChoreographyName().toLowerCase().contains("wp8")) {
				concreteCltsModel.setCltsModel(FileUtils.readFileToString(new File(CLTS_WP8)));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return concreteCltsModel;
	}

	private void mergeRolesToParticipants(List<Participant> result, RolesDefinition roles) {
		for (Role role : roles.getRoles()) {
			Participant participantRole = new Participant();
			participantRole.setName(role.getName());
			if (!result.contains(participantRole)) {
				result.add(participantRole);
			}
		}

	}

	private List<Participant> createParticipant(List<Coord> coords) {
		List<Participant> result = new ArrayList<Participant>();
		for (Coord coord : coords) {
			String[] names = coord.getName().split("-");
			String participantSourceName = names[1];
			String participantTargetName = names[2];
			Participant participant = getParticipant(result, participantSourceName);
			CoordModel coordModel = new CoordModel();
			coordModel.setCdName(coord.getName());
			coordModel.setParticipantTargetName(participantTargetName);
			coordModel.setCoordModel(CoordmetamodelParser.getCoordContent(coord));
			participant.getCoordModels().add(coordModel);
		}
		return result;
	}

	private Participant getParticipant(List<Participant> participants, String participantName) {
		Participant newParticipant = new Participant();
		newParticipant.setName(participantName);
		if (!participants.contains(newParticipant)) {
			participants.add(newParticipant);
			return newParticipant;
		} else {
			return participants.get(participants.indexOf(newParticipant));

		}
	}

	private String createTempFolder() {
		String tempFolder = getTempFolder();
		File tempFolderDir = new File(tempFolder);
		try {
			FileUtils.forceMkdir(tempFolderDir);
			return tempFolder;
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				FileUtils.deleteDirectory(tempFolderDir);
			} catch (IOException e) {
			}
		}
		return "";
	}

	private String getTempFolder() {
		return FileUtils.getTempDirectoryPath() + File.separator + getRandomDir() + File.separator;
	}

	private String getRandomDir() {
		return new Date().getTime() + "";
	}

}
