package it.univaq.di.choreos.synthesis.m2mtransformator.rest;

import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ChoreographyBPMNSpecification;
import it.univaq.di.choreos.synthesis.datamodel.Participant;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformator;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformatorException;
import it.univaq.di.choreos.synthesis.m2mtransformator.impl.M2MTransformatorImpl;

import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class RestM2MTransformator {
	
	private static Logger logger = LoggerFactory.getLogger(RestM2MTransformator.class);
	
	private M2MTransformator processor = new M2MTransformatorImpl();
	
	@Path("/bpmn2clts")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response bpmn2clts(ChoreographyBPMNSpecification bpmnchoreographyspecification) {
		if (bpmnchoreographyspecification == null || "".equals(bpmnchoreographyspecification.getBpmnSpecification())) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			CLTSModel result = processor.bpmn2clts(bpmnchoreographyspecification);
			return Response.ok(result).build();
		} catch (M2MTransformatorException  e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}
	
	
	
	@Path("/clts2coords")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response clts2coords(CLTSModel cltsModel) {
		if (cltsModel == null) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			List<Participant> participants = processor.clts2coords(cltsModel);
			Participants result = new Participants();
			result.setParticipants(participants);
			return Response.ok(result).build();
		} catch (M2MTransformatorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}
	}
	
	@Path("/abstract2ConcreteCLTSModel")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response abstract2ConcreteCLTSModel(Abstract2ConcreteCLTSModel abstract2ConcreteCLTSModel) {
		try {
			CLTSModel result = processor.abstract2ConcreteCLTSModel(abstract2ConcreteCLTSModel.getCltsModel(), abstract2ConcreteCLTSModel.getParticipants());
			return Response.ok(result).build();
		} catch (M2MTransformatorException  e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}

}
