package it.univaq.di.choreos.synthesis.synthesisprocessor.rest;

import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulator;
import it.univaq.di.choreos.synthesis.behavioursimulator.rest.client.BehaviourSimulatorClient;
import it.univaq.di.choreos.synthesis.cdgenerator.CDGenerator;
import it.univaq.di.choreos.synthesis.cdgenerator.rest.client.CDGeneratorClient;
import it.univaq.di.choreos.synthesis.configuration.Configuration;
import it.univaq.di.choreos.synthesis.datamodel.ChoreographyBPMNSpecification;
import it.univaq.di.choreos.synthesis.datamodel.SynthesisResult;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformator;
import it.univaq.di.choreos.synthesis.m2mtransformator.rest.client.M2MTransformatorClient;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscovery;
import it.univaq.di.choreos.synthesis.servicediscovery.rest.client.ServiceDiscoveryClient;
import it.univaq.di.choreos.synthesis.synthesisprocessor.SynthesisProcessor;
import it.univaq.di.choreos.synthesis.synthesisprocessor.SynthesisProcessorException;
import it.univaq.di.choreos.synthesis.synthesisprocessor.impl.SynthesisProcessorImpl;

import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.ow2.choreos.chors.ChoreographyDeployer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("synthesis")
public class RestSynthesisProcessor {

	
	private static Logger logger = LoggerFactory.getLogger(RestSynthesisProcessor.class);

	private static final String BEHAVIOUR_SIMULATOR_HOST;
	private static final String CD_GENERATOR_HOST;
	private static final String M2M_TRANSFORMATOR_HOST;
	private static final String SERVICE_DISCOVERY_HOST;
	private static final String CHOREOGRAPHY_DEPLOYER_HOST;
	
	static {
		BEHAVIOUR_SIMULATOR_HOST = Configuration.get("BEHAVIOUR_SIMULATOR_HOST");
		CD_GENERATOR_HOST = Configuration.get("CD_GENERATOR_HOST");
		M2M_TRANSFORMATOR_HOST = Configuration.get("M2M_TRANSFORMATOR_HOST");
		SERVICE_DISCOVERY_HOST = Configuration.get("SERVICE_DISCOVERY_HOST");
		CHOREOGRAPHY_DEPLOYER_HOST = Configuration.get("CHOREOGRAPHY_DEPLOYER_HOST");
		
	}

	private BehaviourSimulator behaviourSimulator;
	private CDGenerator cdGenerator;
	private M2MTransformator m2mTransformator;
	private ServiceDiscovery serviceDiscovery;
	private ChoreographyDeployer choreographyDeployer;
	private SynthesisProcessor processor;

	public RestSynthesisProcessor() {
		behaviourSimulator = new BehaviourSimulatorClient(BEHAVIOUR_SIMULATOR_HOST);
		cdGenerator = new CDGeneratorClient(CD_GENERATOR_HOST);
		m2mTransformator = new M2MTransformatorClient(M2M_TRANSFORMATOR_HOST);
		serviceDiscovery = new ServiceDiscoveryClient(SERVICE_DISCOVERY_HOST);
		//choreographyDeployer = new ChorDeployerClient(CHOREOGRAPHY_DEPLOYER_HOST);
		choreographyDeployer = new DummyChoreographyDeployer("1");
		processor = new SynthesisProcessorImpl(behaviourSimulator, cdGenerator, m2mTransformator, serviceDiscovery, choreographyDeployer);

	}

	@POST
	@Consumes(MediaType.APPLICATION_XML)
	public Response startSynthesis(ChoreographyBPMNSpecification bpmnSpecification) {

		if (bpmnSpecification == null || bpmnSpecification.getBpmnSpecification() == null || "".equals(bpmnSpecification.getBpmnSpecification())) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			SynthesisResult result = processor.startSynthesis(bpmnSpecification);
			
			return Response.ok(result).build();
		} catch (SynthesisProcessorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}
}
