package it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.wp6;

import it.univaq.di.choreos.synthesis.datamodel.Service;
import it.univaq.di.choreos.synthesis.datamodel.ServiceDiscoveryInput;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscovery;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscoveryException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;

public class WP6ServiceDiscovery implements ServiceDiscovery {
	private static final String[][] resourceService= 
		   {{"airport", "airport"+File.separator+"Airport.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airport-service.jar"},
			{"airportbuscompany", "airportbuscompany"+File.separator+"AirportBusCompany.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportbuscompany-service.jar"},
			{"airportdisplayactuatorsaggregator", "airportdisplayactuatorsaggregator"+File.separator+"AirportDisplayActuatorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportdisplayactuatorsaggregator-service.jar"},
			{"airportinfraredsensorsaggregator", "airportinfraredsensorsaggregator"+File.separator+"AirportInfraredSensorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportinfraredsensorsaggregator-service.jar"},
			{"airportnoisesensorsaggregator", "airportnoisesensorsaggregator"+File.separator+"AirportNoiseSensorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportnoisesensorsaggregator-service.jar"},
			{"airportpressuresensorsaggregator", "airportpressuresensorsaggregator"+File.separator+"AirportPressureSensorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportpressuresensorsaggregator-service.jar"},
			{"airportsignactuatorsaggregator", "airportsignactuatorsaggregator"+File.separator+"AirportSignActuatorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportsignactuatorsaggregator-service.jar"},
			{"airportspeakeractuatorsaggregator", "airportspeakeractuatorsaggregator"+File.separator+"AirportSpeakerActuatorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/airportspeakeractuatorsaggregator-service.jar"},
			{"airporttrafficcontrol", "airtrafficcontrol"+File.separator+"AirTrafficControl.wsdl", "airtrafficcontrolServiceURI"},
			{"bookableamenity", "bookableamenity"+File.separator+"BookableAmenity.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/bookableamenity-service.jar"},
			{"luggagehandlingcompany", "luggagehandlingcompany"+File.separator+"LuggageHandlingCompany.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/luggagehandlingcompany-service.jar"},
			{"middisplayactuatorsaggregator", "middisplayactuatorsaggregator"+File.separator+"MIDDisplayActuatorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/middisplayactuatorsaggregator-service.jar"},
			{"midlocationsensorsaggregator", "midlocationsensorsaggregator"+File.separator+"MIDLocationSensorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/midlocationsensorsaggregator-service.jar"},
			{"midmicrophonesensorsaggregator", "midmicrophonesensorsaggregator"+File.separator+"MIDMicrophoneSensorsAggregator.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/midmicrophonesensorsaggregator-service.jar"},
			{"securitycompany", "securitycompany"+File.separator+"SecurityCompany.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/securitycompany-service.jar"},
			{"standandgatemanagement", "standandgatemanagement"+File.separator+"StandAndGateManagement.wsdl", "http://sd-49168.dedibox.fr/DeployableServices/standandgatemanagement-service.jar"}
			};
	

	private Map<String, List<Service>> services;
	private String resourcesWsdl = "." + File.separator + "src" + File.separator + "test" + File.separator + "resources" + File.separator + "wp6"+ File.separator + "wsdl-input"+ File.separator;
	
	public WP6ServiceDiscovery() {
		services = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceService.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			File wsdlFile = new File(resourcesWsdl+resourceService[i][1]);
			try {
				service.setWsdl(FileUtils.readFileToString(wsdlFile));
				service.setServiceURI(resourceService[i][2]);
				servicesDiscovery.add(service);
				services.put(resourceService[i][0], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
	}

	public List<Service> discoverServices(ServiceDiscoveryInput serviceDiscoveryInput) throws ServiceDiscoveryException {
		return services.get(serviceDiscoveryInput.getParticipant());
	}

}
