package it.univaq.di.choreos.synthesis.synthesisprocessor.impl;

import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulator;
import it.univaq.di.choreos.synthesis.cdgenerator.CDGenerator;
import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ChoreographyBPMNSpecification;
import it.univaq.di.choreos.synthesis.datamodel.CoordModel;
import it.univaq.di.choreos.synthesis.datamodel.Participant;
import it.univaq.di.choreos.synthesis.datamodel.Service;
import it.univaq.di.choreos.synthesis.datamodel.SynthesisResult;
import it.univaq.di.choreos.synthesis.m2mtransformator.M2MTransformator;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscovery;
import it.univaq.di.choreos.synthesis.synthesisprocessor.SynthesisProcessor;
import it.univaq.di.choreos.synthesis.synthesisprocessor.SynthesisProcessorException;
import it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.test01.DummyBehaviourSimulator;
import it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.test01.DummyCDGenerator;
import it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.test01.DummyChoreographyDeployer;
import it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.test01.DummyM2MTransformator;
import it.univaq.di.choreos.synthesis.synthesisprocessor.dummies.test01.DummyServiceDiscovery;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;

import org.junit.Before;
import org.junit.Test;
import org.ow2.choreos.chors.ChoreographyNotFoundException;

public class SynthesisProcessorImplTest {
	private BehaviourSimulator behaviourSimulator;
	private CDGenerator cdGenerator;
	private M2MTransformator m2mTransformator;
	private ServiceDiscovery serviceDiscovery;
	private DummyChoreographyDeployer choreographyDeployer;
	private SynthesisProcessor synthesisProcessor;

	
	@Before
	public void setUp() {
		
	}

	@Test
	public void test01() throws SynthesisProcessorException, ChoreographyNotFoundException, JAXBException, IOException {
		CLTSModel cltsModel = new CLTSModel();
		List<Participant> participants = new ArrayList<Participant>();
		Participant p1 = new Participant();
		p1.setName("P1");
		CoordModel cdp1 = new CoordModel();
		cdp1.setCdName("CD-P1-P2");
		cdp1.setParticipantTargetName("P2");
		cdp1.setCoordModel("");
		p1.getCoordModels().add(cdp1);

		Participant p2 = new Participant();
		p2.setName("P2");
		CoordModel cdp2 = new CoordModel();
		cdp2.setCdName("CD-P2-P3");
		cdp2.setParticipantTargetName("P3");
		cdp2.setCoordModel("");
		p2.getCoordModels().add(cdp2);

		Participant p3 = new Participant();
		p3.setName("P3");
		participants.add(p1);
		participants.add(p2);
		participants.add(p3);

		m2mTransformator = new DummyM2MTransformator(cltsModel, participants);

		boolean simulationResult = true;
		behaviourSimulator = new DummyBehaviourSimulator(simulationResult);

		Map<String, List<Service>> services = new HashMap<String, List<Service>>();
		List<Service> servicesP1 = new ArrayList<Service>();
		Service serviceP1 = new Service();
		serviceP1.setServiceURI("P1");
		serviceP1.setWsdl("wsdl_P1");
		servicesP1.add(serviceP1);
		services.put(p1.getName(), servicesP1);

		List<Service> servicesP2 = new ArrayList<Service>();
		Service serviceP2 = new Service();
		serviceP2.setServiceURI("P2");
		serviceP2.setWsdl("wsdl_P2");
		servicesP2.add(serviceP2);
		services.put(p2.getName(), servicesP2);

		List<Service> servicesP3 = new ArrayList<Service>();
		Service serviceP3 = new Service();
		serviceP3.setServiceURI("P3");
		serviceP3.setWsdl("wsdl_P3");
		servicesP3.add(serviceP3);
		services.put(p3.getName(), servicesP3);

		serviceDiscovery = new DummyServiceDiscovery(services);
		String urlCD = "http://localhost";
		cdGenerator = new DummyCDGenerator(urlCD);
		choreographyDeployer = new DummyChoreographyDeployer("1");

		synthesisProcessor = new SynthesisProcessorImpl(behaviourSimulator, cdGenerator, m2mTransformator, serviceDiscovery, choreographyDeployer);

		ChoreographyBPMNSpecification bpmnSpecification = new ChoreographyBPMNSpecification();
		SynthesisResult result = synthesisProcessor.startSynthesis(bpmnSpecification);

		org.junit.Assert.assertTrue(result.isResult());
		String choreographyDeployerString = result.getChorSpec();

		// String chorSpec = FileUtils.readFileToString(new File(resourcesTest +
		// "ChorSpec.xml"));
		System.out.println(choreographyDeployerString);
		// org.junit.Assert.assertEquals(chorSpec.trim().replaceAll("[\n\r]",
		// ""), choreographyDeployerString.trim().replaceAll("[\n\r]", ""));

	}
}
